"""Add EnergyPlus objects only for VRFsystem."""


def addOpTempTherm(self, verboseMode : bool = True):
    """
    Amend ZoneControl:Thermostat:OperativeTemperature objects.

    Add ZoneControl:Thermostat:OperativeTemperature objects for each zone.
    """
    for zonename_orig in self.zonenames_orig:
        if zonename_orig+' Thermostat' in [thermostat.Thermostat_Name
                                           for thermostat
                                           in self.idf1.idfobjects
                                           ['ZoneControl:Thermostat:OperativeTemperature']]:
            if verboseMode:
                print(zonename_orig+' Thermostat already was in the model')
        else:
            self.idf1.newidfobject(
                'ZoneControl:Thermostat:OperativeTemperature',
                Thermostat_Name=zonename_orig+" Thermostat",
                Radiative_Fraction_Input_Mode="Scheduled",
                Fixed_Radiative_Fraction='',
                Radiative_Fraction_Schedule_Name='TypOperativeTempControlSch'
                )
            if verboseMode:
                print(zonename_orig+' Thermostat has been added')


def addBaseSchedules(self, verboseMode : bool = True):
    """
    Amend Schedule:Compact objects.

    Checks Schedule:Compact objects needed for accim,
    and add them in case these are not in the model
    """
    if "On" in [schedule.Name for schedule in self.idf1.idfobjects['Schedule:Compact']]:
        if verboseMode:
            print("On Schedule already was in the model")
    else:
        self.idf1.newidfobject(
            'Schedule:Compact',
            Name="On",
            Schedule_Type_Limits_Name="Any Number",
            Field_1='Through: 12/31',
            Field_2='For: AllDays',
            Field_3='Until: 24:00,1'
            )
        if verboseMode:
            print("On Schedule has been added")
    
    if "TypOperativeTempControlSch" in [schedule.Name for schedule in self.idf1.idfobjects['Schedule:Compact']]:
        if verboseMode:
            print("TypOperativeTempControlSch Schedule already was in the model")
    else:
        self.idf1.newidfobject(
            'Schedule:Compact',
            Name="TypOperativeTempControlSch",
            Schedule_Type_Limits_Name="Temperature",
            Field_1='Through: 12/31',
            Field_2='For: SummerDesignDay WinterDesignDay',
            Field_3='Until: 24:00,0',
            Field_4='For: AllOtherDays',
            Field_5='Until: 24:00, 0.5',
            )
        if verboseMode:
            print("TypOperativeTempControlSch Schedule has been added")


def setAvailSchOn(self, verboseMode: bool = True):
    """
    Amend availability schedules.

    Assign On Compact:Schedule to heating and cooling availability
    schedule names.
    """
    for schedule in [i for i in self.idf1.idfobjects['ZoneHVAC:IdealLoadsAirSystem']]:
        schedule.Heating_Availability_Schedule_Name='On'
        schedule.Cooling_Availability_Schedule_Name='On'
    if verboseMode:
        print('All ZoneHVAC:IdealLoadsAirSystem '
              'Heating and Cooling availability schedules has been set to on')


def addVRFsystemSch(self, verboseMode: bool = True):
    """
    Amend Schedule:Compact objects for VRFsystem.

    Add Schedule:Compact objects needed for VRFsystem to work,
    other than AHST_Sch and ACST_Sch Schedules.
    """
    addVRFsystemSch_dict = {
        'On 24/7': 'Until: 24:00,1',
        'Control type schedule: Always 4': 'Until: 24:00,4',
        'Relative humidity setpoint schedule: Always 50.00': 'Until: 24:00,50',
        'Heating Fanger comfort setpoint: Always -0.5': 'Until: 24:00,-0.5',
        'Cooling Fanger comfort setpoint: Always  0.1': 'Until: 24:00, 0.1',
        'Zone CO2 setpoint: Always 900ppm': 'Until: 24:00, 900',
        'Min CO2 concentration: Always 600ppm': 'Until: 24:00, 600',
        'Generic contaminant setpoint: Always 0.5ppm': 'Until: 24:00, 0.5',
        'Air distribution effectiveness (always 1)': 'Until: 24:00, 1'
        }
    for i in addVRFsystemSch_dict:
        if i in [schedule.Name for schedule
                 in self.idf1.idfobjects['Schedule:Compact']]:
            if verboseMode:
                print(i+" Schedule already was in the model")
        else:
            self.idf1.newidfobject('Schedule:Compact',
                                   Name=i,
                                   Schedule_Type_Limits_Name="Any Number",
                                   Field_1='Through: 12/31',
                                   Field_2='For: AllDays',
                                   Field_3=addVRFsystemSch_dict[i])
            if verboseMode:
                print(i+" Schedule has been added")

    if "VRF Heating Cooling (Northern Hemisphere)" in [schedule.Name
                                                       for schedule
                                                       in self.idf1.idfobjects
                                                       ['Schedule:Compact']]:
        if verboseMode:
            print("VRF Heating Cooling (Northern Hemisphere) Schedule already was in the model")
    else:
        self.idf1.newidfobject(
            'Schedule:Compact',
            Name="VRF Heating Cooling (Northern Hemisphere)",
            Schedule_Type_Limits_Name="Any Number",
            Field_1='Through: 31 Mar',
            Field_2='For: AllDays',
            Field_3='Until: 24:00, 0',
            Field_4='Through: 30 Sep',
            Field_5='For: AllDays',
            Field_6='Until: 24:00, 1',
            Field_7='Through: 31 Dec',
            Field_8='For: AllDays',
            Field_9='Until: 24:00, 0')
        if verboseMode:
            print("VRF Heating Cooling (Northern Hemisphere) Schedule has been added")


def addCurveObj(self, verboseMode: bool = True):
    """Add Curve Objects needed for VRFsystem to work."""
    # curvecubiclist=([i for i in self.idf1.idfobjects['Curve:Cubic']])
    # print(curvecubiclist)

    if 'DefaultFanEffRatioCurve' in [i.Name for i in self.idf1.idfobjects['Curve:Cubic']]:
        if verboseMode:
            print('DefaultFanEffRatioCurve Curve:Cubic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Cubic',
            Name='DefaultFanEffRatioCurve',
            Coefficient1_Constant='0.33856828',
            Coefficient2_x='1.72644131',
            Coefficient3_x2='-1.49280132',
            Coefficient4_x3='0.42776208',
            Minimum_Value_of_x='0.5',
            Maximum_Value_of_x='1.5',
            Minimum_Curve_Output='0.3',
            Maximum_Curve_Output='1.0',
            Input_Unit_Type_for_X='',
            Output_Unit_Type=''
            )
        if verboseMode:
            print("DefaultFanEffRatioCurve Curve:Cubic Object has been added")

    if 'VRFTUCoolCapFT' in [i.Name for i in self.idf1.idfobjects['Curve:Cubic']]:
        if verboseMode:
            print('VRFTUCoolCapFT Curve:Cubic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Cubic',
            Name='VRFTUCoolCapFT',
            Coefficient1_Constant='0.504547273506488',
            Coefficient2_x='0.0288891279198444',
            Coefficient3_x2='-0.000010819418650677',
            Coefficient4_x3='0.0000101359395177008',
            Minimum_Value_of_x='0.0',
            Maximum_Value_of_x='50.0',
            Minimum_Curve_Output='0.5',
            Maximum_Curve_Output='1.5',
            Input_Unit_Type_for_X='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRFTUCoolCapFT Curve:Cubic Object has been added")

    if 'VRFTUHeatCapFT' in [i.Name for i in self.idf1.idfobjects['Curve:Cubic']]:
        if verboseMode:
            print('VRFTUHeatCapFT Curve:Cubic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Cubic',
            Name='VRFTUHeatCapFT',
            Coefficient1_Constant='-0.390708928227928',
            Coefficient2_x='0.261815023760162',
            Coefficient3_x2='-0.0130431603151873',
            Coefficient4_x3='0.000178131745997821',
            Minimum_Value_of_x='0.0',
            Maximum_Value_of_x='50.0',
            Minimum_Curve_Output='0.5',
            Maximum_Curve_Output='1.5',
            Input_Unit_Type_for_X='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRFTUHeatCapFT Curve:Cubic Object has been added")

    if 'VRFCoolCapFTBoundary' in [i.Name for i in self.idf1.idfobjects['Curve:Cubic']]:
        if verboseMode:
            print('VRFCoolCapFTBoundary Curve:Cubic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Cubic',
            Name='VRFCoolCapFTBoundary',
            Coefficient1_Constant='25.73473775',
            Coefficient2_x='-0.03150043',
            Coefficient3_x2='-0.01416595',
            Coefficient4_x3='0',
            Minimum_Value_of_x='11',
            Maximum_Value_of_x='30',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Output_Unit_Type=''
            )
        if verboseMode:
            print("VRFCoolCapFTBoundary Curve:Cubic Object has been added")

    if 'VRFCoolEIRFTBoundary' in [i.Name for i in self.idf1.idfobjects['Curve:Cubic']]:
        print('VRFCoolEIRFTBoundary Curve:Cubic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Cubic',
            Name='VRFCoolEIRFTBoundary',
            Coefficient1_Constant='25.73473775',
            Coefficient2_x='-0.03150043',
            Coefficient3_x2='-0.01416595',
            Coefficient4_x3='0',
            Minimum_Value_of_x='15',
            Maximum_Value_of_x='24',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Output_Unit_Type=''
            )
        if verboseMode:
            print("VRFCoolEIRFTBoundary Curve:Cubic Object has been added")

    if 'CoolingEIRLowPLR' in [i.Name for i in self.idf1.idfobjects['Curve:Cubic']]:
        if verboseMode:
            print('CoolingEIRLowPLR Curve:Cubic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Cubic',
            Name='CoolingEIRLowPLR',
            Coefficient1_Constant='0.4628123',
            Coefficient2_x='-1.0402406',
            Coefficient3_x2='2.17490997',
            Coefficient4_x3='-0.5974817',
            Minimum_Value_of_x='0',
            Maximum_Value_of_x='1',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Output_Unit_Type='Capacity'
            )
        if verboseMode:
            print("CoolingEIRLowPLR Curve:Cubic Object has been added")

    if 'VRFHeatCapFTBoundary' in [i.Name for i in self.idf1.idfobjects['Curve:Cubic']]:
        if verboseMode:
            print('VRFHeatCapFTBoundary Curve:Cubic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Cubic',
            Name='VRFHeatCapFTBoundary',
            Coefficient1_Constant='-7.6000882',
            Coefficient2_x='3.05090016',
            Coefficient3_x2='-0.1162844',
            Coefficient4_x3='0.0',
            Minimum_Value_of_x='15',
            Maximum_Value_of_x='27',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Output_Unit_Type=''
            )
        if verboseMode:
            print("VRFHeatCapFTBoundary Curve:Cubic Object has been added")

    if 'VRFHeatEIRFTBoundary' in [i.Name for i in self.idf1.idfobjects['Curve:Cubic']]:
        if verboseMode:
            print('VRFHeatEIRFTBoundary Curve:Cubic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Cubic',
            Name='VRFHeatEIRFTBoundary',
            Coefficient1_Constant='-7.6000882',
            Coefficient2_x='3.05090016',
            Coefficient3_x2='-0.1162844',
            Coefficient4_x3='0.0',
            Minimum_Value_of_x='15',
            Maximum_Value_of_x='27',
            Minimum_Curve_Output='-20',
            Maximum_Curve_Output='15',
            Input_Unit_Type_for_X='Temperature',
            Output_Unit_Type=''
            )
        if verboseMode:
            print("VRFHeatEIRFTBoundary Curve:Cubic Object has been added")

    if 'HeatingEIRLowPLR' in [i.Name for i in self.idf1.idfobjects['Curve:Cubic']]:
        if verboseMode:
            print('HeatingEIRLowPLR Curve:Cubic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Cubic',
            Name='HeatingEIRLowPLR',
            Coefficient1_Constant='0.1400093',
            Coefficient2_x='0.6415002',
            Coefficient3_x2='0.1339047',
            Coefficient4_x3='0.0845859',
            Minimum_Value_of_x='0',
            Maximum_Value_of_x='1',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Dimensionless',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("HeatingEIRLowPLR Curve:Cubic Object has been added")

    # curveexponentlist=[i for i in self.idf1.idfobjects['Curve:Exponent']]
    # print(curveexponentlist)

    if 'DefaultFanPowerRatioCurve' in [i.Name for i in self.idf1.idfobjects['Curve:Exponent']]:
        if verboseMode:
            print('DefaultFanPowerRatioCurve Curve:Exponent Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Exponent',
            Name='DefaultFanPowerRatioCurve',
            Coefficient1_Constant='0',
            Coefficient2_Constant='1',
            Coefficient3_Constant='3',
            Minimum_Value_of_x='0',
            Maximum_Value_of_x='1.5',
            Minimum_Curve_Output='0.01',
            Maximum_Curve_Output='1.5',
            Input_Unit_Type_for_X='',
            Output_Unit_Type=''
            )
        if verboseMode:
            print("DefaultFanPowerRatioCurve Curve:Exponent Object has been added")

    # curvebiquadraticlist=[i for i in self.idf1.idfobjects['Curve:Biquadratic']]
    # print(curvebiquadraticlist)

    if 'DXHtgCoilDefrostEIRFT' in [i.Name for i in self.idf1.idfobjects['Curve:Biquadratic']]:
        if verboseMode:
            print('DXHtgCoilDefrostEIRFT Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='DXHtgCoilDefrostEIRFT',
            Coefficient1_Constant='1.0',
            Coefficient2_x='0.0',
            Coefficient3_x2='0.0',
            Coefficient4_y='0.0',
            Coefficient5_y2='0',
            Coefficient6_xy='0',
            Minimum_Value_of_x='0.0',
            Maximum_Value_of_x='50.0',
            Minimum_Value_of_y='0.0',
            Maximum_Value_of_y='50.0',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("DXHtgCoilDefrostEIRFT Curve:Biquadratic Object has been added")

    if 'VRFCoolCapFT' in [i.Name for i in self.idf1.idfobjects['Curve:Biquadratic']]:
        if verboseMode:
            print('VRFCoolCapFT Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='VRFCoolCapFT',
            Coefficient1_Constant='0.576882692',
            Coefficient2_x='0.017447952',
            Coefficient3_x2='0.000583269',
            Coefficient4_y='-1.76324E-06',
            Coefficient5_y2='-7.474E-09',
            Coefficient6_xy='-1.30413E-07',
            Minimum_Value_of_x='15',
            Maximum_Value_of_x='24',
            Minimum_Value_of_y='-5',
            Maximum_Value_of_y='23',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRFCoolCapFT Curve:Biquadratic Object has been added")

    if 'VRFCoolCapFTHi' in [i.Name for i in self.idf1.idfobjects['Curve:Biquadratic']]:
        if verboseMode:
            print('VRFCoolCapFTHi Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='VRFCoolCapFTHi',
            Coefficient1_Constant='0.6867358',
            Coefficient2_x='0.0207631',
            Coefficient3_x2='0.0005447',
            Coefficient4_y='-0.0016218',
            Coefficient5_y2='-4.259E-07',
            Coefficient6_xy='-0.0003392',
            Minimum_Value_of_x='15',
            Maximum_Value_of_x='24',
            Minimum_Value_of_y='16',
            Maximum_Value_of_y='43',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRFCoolCapFTHi Curve:Biquadratic Object has been added")

    if 'VRFCoolEIRFT' in [i.Name for i in self.idf1.idfobjects['Curve:Biquadratic']]:
        if verboseMode:
            print('VRFCoolEIRFT Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='VRFCoolEIRFT',
            Coefficient1_Constant='0.989010541',
            Coefficient2_x='-0.02347967',
            Coefficient3_x2='0.000199711',
            Coefficient4_y='0.005968336',
            Coefficient5_y2='-1.0289E-07',
            Coefficient6_xy='-0.00015686',
            Minimum_Value_of_x='15',
            Maximum_Value_of_x='24',
            Minimum_Value_of_y='-5',
            Maximum_Value_of_y='23',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRFCoolEIRFT Curve:Biquadratic Object has been added")

    if 'VRFCoolEIRFTHi' in [i.Name for i in self.idf1.idfobjects['Curve:Biquadratic']]:
        if verboseMode:
            print('VRFCoolEIRFTHi Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='VRFCoolEIRFTHi',
            Coefficient1_Constant='0.14351470',
            Coefficient2_x='0.01860035',
            Coefficient3_x2='-0.0003954',
            Coefficient4_y='0.02485219',
            Coefficient5_y2='0.00016329',
            Coefficient6_xy='-0.0006244',
            Minimum_Value_of_x='15',
            Maximum_Value_of_x='24',
            Minimum_Value_of_y='16',
            Maximum_Value_of_y='43',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRFCoolEIRFTHi Curve:Biquadratic Object has been added")

    if 'VRFHeatCapFT' in [i.Name for i in self.idf1.idfobjects['Curve:Biquadratic']]:
        if verboseMode:
            print('VRFHeatCapFT Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='VRFHeatCapFT',
            Coefficient1_Constant='1.014599599',
            Coefficient2_x='-0.002506703',
            Coefficient3_x2='-0.000141599',
            Coefficient4_y='0.026931595',
            Coefficient5_y2='1.83538E-06',
            Coefficient6_xy='-0.000358147',
            Minimum_Value_of_x='15',
            Maximum_Value_of_x='27',
            Minimum_Value_of_y='-20',
            Maximum_Value_of_y='15',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRFHeatCapFT Curve:Biquadratic Object has been added")

    if 'VRFHeatCapFTHi' in [i.Name for i in self.idf1.idfobjects['Curve:Biquadratic']]:
        if verboseMode:
            print('VRFHeatCapFTHi Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='VRFHeatCapFTHi',
            Coefficient1_Constant='1.161134821',
            Coefficient2_x='0.027478868',
            Coefficient3_x2='-0.00168795',
            Coefficient4_y='0.001783378',
            Coefficient5_y2='2.03208E-06',
            Coefficient6_xy='-6.8969E-05',
            Minimum_Value_of_x='15',
            Maximum_Value_of_x='27',
            Minimum_Value_of_y='-10',
            Maximum_Value_of_y='15',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRFHeatCapFTHi Curve:Biquadratic Object has been added")

    if 'VRFHeatEIRFT' in [i.Name for i in self.idf1.idfobjects['Curve:Biquadratic']]:
        if verboseMode:
            print('VRFHeatEIRFT Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='VRFHeatEIRFT',
            Coefficient1_Constant='0.87465501',
            Coefficient2_x='-0.01319754',
            Coefficient3_x2='0.00110307',
            Coefficient4_y='-0.0133118',
            Coefficient5_y2='0.00089017',
            Coefficient6_xy='-0.00012766',
            Minimum_Value_of_x='15',
            Maximum_Value_of_x='27',
            Minimum_Value_of_y='-20',
            Maximum_Value_of_y='12',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRFHeatEIRFT Curve:Biquadratic Object has been added")

    if 'VRFHeatEIRFTHi' in [i.Name for i in self.idf1.idfobjects['Curve:Biquadratic']]:
        if verboseMode:
            print('VRFHeatEIRFTHi Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='VRFHeatEIRFTHi',
            Coefficient1_Constant='2.504005146',
            Coefficient2_x='-0.05736767',
            Coefficient3_x2='4.07336E-05',
            Coefficient4_y='-0.12959669',
            Coefficient5_y2='0.00135839',
            Coefficient6_xy='0.00317047',
            Minimum_Value_of_x='15',
            Maximum_Value_of_x='27',
            Minimum_Value_of_y='-10',
            Maximum_Value_of_y='15',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRFHeatEIRFTHi Curve:Biquadratic Object has been added")

    if 'CoolingLengthCorrectionFactor' in [i.Name
                                           for i
                                           in self.idf1.idfobjects['Curve:Biquadratic']]:
        if verboseMode:
            print('CoolingLengthCorrectionFactor Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='CoolingLengthCorrectionFactor',
            Coefficient1_Constant='1.0693794',
            Coefficient2_x='-0.0014951',
            Coefficient3_x2='2.56E-06',
            Coefficient4_y='-0.1151104',
            Coefficient5_y2='0.0511169',
            Coefficient6_xy='-0.0004369',
            Minimum_Value_of_x='8',
            Maximum_Value_of_x='175',
            Minimum_Value_of_y='0.5',
            Maximum_Value_of_y='1.5',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("CoolingLengthCorrectionFactor Curve:Biquadratic Object has been added")

    if 'VRF Piping Correction Factor for Length in Heating Mode' in [i.Name
                                                                     for i
                                                                     in self.idf1.idfobjects
                                                                     ['Curve:Biquadratic']]:
        if verboseMode:
            print('VRF Piping Correction Factor for Length in '
                  'Heating Mode Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='VRF Piping Correction Factor for Length in Heating Mode',
            Coefficient1_Constant='.989916',
            Coefficient2_x='.001961',
            Coefficient3_x2='-.000036',
            Coefficient4_y='0',
            Coefficient5_y2='0',
            Coefficient6_xy='0',
            Minimum_Value_of_x='7',
            Maximum_Value_of_x='106.5',
            Minimum_Value_of_y='1',
            Maximum_Value_of_y='1',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Distance',
            Input_Unit_Type_for_Y='Dimensionless',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRF Piping Correction Factor for Length in "
                  "Heating Mode Curve:Biquadratic Object has been added")

    if 'VRF Heat Recovery Cooling Capacity Modifier' in [i.Name
                                                         for i
                                                         in self.idf1.idfobjects
                                                         ['Curve:Biquadratic']]:
        if verboseMode:
            print('VRF Heat Recovery Cooling Capacity Modifier '
                  'Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='VRF Heat Recovery Cooling Capacity Modifier',
            Coefficient1_Constant='.9',
            Coefficient2_x='0',
            Coefficient3_x2='0',
            Coefficient4_y='0',
            Coefficient5_y2='0',
            Coefficient6_xy='0',
            Minimum_Value_of_x='-100',
            Maximum_Value_of_x='100',
            Minimum_Value_of_y='-100',
            Maximum_Value_of_y='100',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRF Heat Recovery Cooling Capacity Modifier "
                  "Curve:Biquadratic Object has been added")

    if 'VRF Heat Recovery Cooling Energy Modifier' in [i.Name
                                                       for i
                                                       in self.idf1.idfobjects
                                                       ['Curve:Biquadratic']]:
        if verboseMode:
            print('VRF Heat Recovery Cooling Energy Modifier '
                  'Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='VRF Heat Recovery Cooling Energy Modifier',
            Coefficient1_Constant='1.1',
            Coefficient2_x='0',
            Coefficient3_x2='0',
            Coefficient4_y='0',
            Coefficient5_y2='0',
            Coefficient6_xy='0',
            Minimum_Value_of_x='-100',
            Maximum_Value_of_x='100',
            Minimum_Value_of_y='-100',
            Maximum_Value_of_y='100',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRF Heat Recovery Cooling Energy Modifier "
                  "Curve:Biquadratic Object has been added")

    if 'VRF Heat Recovery Heating Capacity Modifier' in [i.Name
                                                         for i
                                                         in self.idf1.idfobjects
                                                         ['Curve:Biquadratic']]:
        if verboseMode:
            print('VRF Heat Recovery Heating Capacity Modifier '
                  'Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='VRF Heat Recovery Heating Capacity Modifier',
            Coefficient1_Constant='.9',
            Coefficient2_x='0',
            Coefficient3_x2='0',
            Coefficient4_y='0',
            Coefficient5_y2='0',
            Coefficient6_xy='0',
            Minimum_Value_of_x='-100',
            Maximum_Value_of_x='100',
            Minimum_Value_of_y='-100',
            Maximum_Value_of_y='100',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRF Heat Recovery Heating Capacity Modifier "
                  "Curve:Biquadratic Object has been added")

    if 'VRF Heat Recovery Heating Energy Modifier' in [i.Name
                                                       for i
                                                       in self.idf1.idfobjects
                                                       ['Curve:Biquadratic']]:
        if verboseMode:
            print('VRF Heat Recovery Heating Energy Modifier '
                  'Curve:Biquadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Biquadratic',
            Name='VRF Heat Recovery Heating Energy Modifier',
            Coefficient1_Constant='1.1',
            Coefficient2_x='0',
            Coefficient3_x2='0',
            Coefficient4_y='0',
            Coefficient5_y2='0',
            Coefficient6_xy='0',
            Minimum_Value_of_x='-100',
            Maximum_Value_of_x='100',
            Minimum_Value_of_y='-100',
            Maximum_Value_of_y='100',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='Temperature',
            Input_Unit_Type_for_Y='Temperature',
            Output_Unit_Type='Dimensionless'
            )
        if verboseMode:
            print("VRF Heat Recovery Heating Energy Modifier "
                  "Curve:Biquadratic Object has been added")

    # curvequadraticlist=[i for i in self.idf1.idfobjects['Curve:Quadratic']]
    # print(curvequadraticlist)

    if 'VRFACCoolCapFFF' in [i.Name for i in self.idf1.idfobjects['Curve:Quadratic']]:
        if verboseMode:
            print('VRFACCoolCapFFF Curve:Quadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Quadratic',
            Name='VRFACCoolCapFFF',
            Coefficient1_Constant='0.8',
            Coefficient2_x='0.2',
            Coefficient3_x2='0.0',
            Minimum_Value_of_x='0.5',
            Maximum_Value_of_x='1.5',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='',
            Output_Unit_Type=''
            )
        if verboseMode:
            print("VRFACCoolCapFFF Curve:Quadratic Object has been added")

    if 'CoolingEIRHiPLR' in [i.Name for i in self.idf1.idfobjects['Curve:Quadratic']]:
        if verboseMode:
            print('CoolingEIRHiPLR Curve:Quadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Quadratic',
            Name='CoolingEIRHiPLR',
            Coefficient1_Constant='1.0',
            Coefficient2_x='0.0',
            Coefficient3_x2='0.0',
            Minimum_Value_of_x='1.0',
            Maximum_Value_of_x='1.5',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='',
            Output_Unit_Type=''
            )
        if verboseMode:
            print("CoolingEIRHiPLR Curve:Quadratic Object has been added")

    if 'VRFCPLFFPLR' in [i.Name for i in self.idf1.idfobjects['Curve:Quadratic']]:
        if verboseMode:
            print('VRFCPLFFPLR Curve:Quadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Quadratic',
            Name='VRFCPLFFPLR',
            Coefficient1_Constant='0.85',
            Coefficient2_x='0.15',
            Coefficient3_x2='0.0',
            Minimum_Value_of_x='0.0',
            Maximum_Value_of_x='1.0',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='',
            Output_Unit_Type=''
            )
        if verboseMode:
            print("VRFCPLFFPLR Curve:Quadratic Object has been added")

    if 'HeatingEIRHiPLR' in [i.Name for i in self.idf1.idfobjects['Curve:Quadratic']]:
        if verboseMode:
            print('HeatingEIRHiPLR Curve:Quadratic Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Quadratic',
            Name='HeatingEIRHiPLR',
            Coefficient1_Constant='2.4294355',
            Coefficient2_x='-2.235887',
            Coefficient3_x2='0.8064516',
            Minimum_Value_of_x='1.0',
            Maximum_Value_of_x='1.5',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='',
            Output_Unit_Type=''
            )
        if verboseMode:
            print("HeatingEIRHiPLR Curve:Quadratic Object has been added")

    # curvelinearlist=[i for i in self.idf1.idfobjects['Curve:Linear']]
    # print(curvelinearlist)

    if 'CoolingCombRatio' in [i.Name for i in self.idf1.idfobjects['Curve:Linear']]:
        if verboseMode:
            print('CoolingCombRatio Curve:Linear Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Linear',
            Name='CoolingCombRatio',
            Coefficient1_Constant='0.618055',
            Coefficient2_x='0.381945',
            Minimum_Value_of_x='1.0',
            Maximum_Value_of_x='1.5',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='',
            Output_Unit_Type=''
            )
        if verboseMode:
            print("CoolingCombRatio Curve:Linear Object has been added")

    if 'HeatingCombRatio' in [i.Name for i in self.idf1.idfobjects['Curve:Linear']]:
        if verboseMode:
            print('HeatingCombRatio Curve:Linear Object already was in the model')
    else:
        self.idf1.newidfobject(
            'Curve:Linear',
            Name='HeatingCombRatio',
            Coefficient1_Constant='0.96034',
            Coefficient2_x='0.03966',
            Minimum_Value_of_x='1.0',
            Maximum_Value_of_x='1.5',
            Minimum_Curve_Output='',
            Maximum_Curve_Output='',
            Input_Unit_Type_for_X='',
            Output_Unit_Type=''
            )
        if verboseMode:
            print("HeatingCombRatio Curve:Linear Object has been added")


def addDetHVACobj(self, EnergyPlus_version: str = None, verboseMode: bool = True, TempCtrl: str = None):
    """Add Detailed HVAC objects for VRFsystem to work."""
    for zn in self.zonenames_orig:
        if 'VRF Outdoor Unit_'+zn in [i.Heat_Pump_Name
                                      for i
                                      in self.idf1.idfobjects
                                      ['AirConditioner:VariableRefrigerantFlow']]:
            if verboseMode:
                print('VRF Outdoor Unit_'+
                      zn+
                      ' AirConditioner:VariableRefrigerantFlow '
                      'Object already was in the model')
        else:
            if EnergyPlus_version.lower() == '9.6' or '22' in EnergyPlus_version.lower():
                self.idf1.newidfobject(
                    'AirConditioner:VariableRefrigerantFlow',
                    Heat_Pump_Name='VRF Outdoor Unit_'+zn,
                    Availability_Schedule_Name='On 24/7',
                    Gross_Rated_Total_Cooling_Capacity='autosize',
                    Gross_Rated_Cooling_COP=2,
                    Minimum_Condenser_Inlet_Node_Temperature_in_Cooling_Mode=-6,
                    Maximum_Condenser_Inlet_Node_Temperature_in_Cooling_Mode=43,
                    # Minimum_Outdoor_Temperature_in_Cooling_Mode=-6,
                    # Maximum_Outdoor_Temperature_in_Cooling_Mode=43,
                    Cooling_Capacity_Ratio_Modifier_Function_of_Low_Temperature_Curve_Name=
                    'VRFCoolCapFT',
                    Cooling_Capacity_Ratio_Boundary_Curve_Name='VRFCoolCapFTBoundary',
                    Cooling_Capacity_Ratio_Modifier_Function_of_High_Temperature_Curve_Name=
                    'VRFCoolCapFTHi',
                    Cooling_Energy_Input_Ratio_Modifier_Function_of_Low_Temperature_Curve_Name=
                    'VRFCoolEIRFT',
                    Cooling_Energy_Input_Ratio_Boundary_Curve_Name='VRFCoolEIRFTBoundary',
                    Cooling_Energy_Input_Ratio_Modifier_Function_of_High_Temperature_Curve_Name=
                    'VRFCoolEIRFTHi',
                    Cooling_Energy_Input_Ratio_Modifier_Function_of_Low_PartLoad_Ratio_Curve_Name=
                    'CoolingEIRLowPLR',
                    Cooling_Energy_Input_Ratio_Modifier_Function_of_High_PartLoad_Ratio_Curve_Name=
                    'CoolingEIRHiPLR',
                    Cooling_Combination_Ratio_Correction_Factor_Curve_Name=
                    'CoolingCombRatio',
                    Cooling_PartLoad_Fraction_Correlation_Curve_Name='VRFCPLFFPLR',
                    Gross_Rated_Heating_Capacity='autosize',
                    Rated_Heating_Capacity_Sizing_Ratio=1,
                    Gross_Rated_Heating_COP=2.1,
                    Minimum_Condenser_Inlet_Node_Temperature_in_Heating_Mode=-20,
                    Maximum_Condenser_Inlet_Node_Temperature_in_Heating_Mode=40,
                    Heating_Capacity_Ratio_Modifier_Function_of_Low_Temperature_Curve_Name=
                    'VRFHeatCapFT',
                    Heating_Capacity_Ratio_Boundary_Curve_Name='VRFHeatCapFTBoundary',
                    Heating_Capacity_Ratio_Modifier_Function_of_High_Temperature_Curve_Name=
                    'VRFHeatCapFTHi',
                    Heating_Energy_Input_Ratio_Modifier_Function_of_Low_Temperature_Curve_Name=
                    'VRFHeatEIRFT',
                    Heating_Energy_Input_Ratio_Boundary_Curve_Name='VRFHeatEIRFTBoundary',
                    Heating_Energy_Input_Ratio_Modifier_Function_of_High_Temperature_Curve_Name=
                    'VRFHeatEIRFTHi',
                    Heating_Performance_Curve_Outdoor_Temperature_Type='WetBulbTemperature',
                    Heating_Energy_Input_Ratio_Modifier_Function_of_Low_PartLoad_Ratio_Curve_Name=
                    'HeatingEIRLowPLR',
                    Heating_Energy_Input_Ratio_Modifier_Function_of_High_PartLoad_Ratio_Curve_Name=
                    'HeatingEIRHiPLR',
                    Heating_Combination_Ratio_Correction_Factor_Curve_Name=
                    'HeatingCombRatio',
                    Heating_PartLoad_Fraction_Correlation_Curve_Name='VRFCPLFFPLR',
                    Minimum_Heat_Pump_PartLoad_Ratio=0.2,
                    Zone_Name_for_Master_Thermostat_Location='',
                    Master_Thermostat_Priority_Control_Type='LoadPriority',
                    Thermostat_Priority_Schedule_Name='',
                    Zone_Terminal_Unit_List_Name='VRF Outdoor Unit_'+zn+' Zone List',
                    Heat_Pump_Waste_Heat_Recovery='Yes',
                    Equivalent_Piping_Length_used_for_Piping_Correction_Factor_in_Cooling_Mode=50,
                    Vertical_Height_used_for_Piping_Correction_Factor=15,
                    Piping_Correction_Factor_for_Length_in_Cooling_Mode_Curve_Name=
                    'CoolingLengthCorrectionFactor',
                    Piping_Correction_Factor_for_Height_in_Cooling_Mode_Coefficient=0,
                    Equivalent_Piping_Length_used_for_Piping_Correction_Factor_in_Heating_Mode=50,
                    Piping_Correction_Factor_for_Length_in_Heating_Mode_Curve_Name=
                    'VRF Piping Correction Factor for Length in Heating Mode',
                    Piping_Correction_Factor_for_Height_in_Heating_Mode_Coefficient=0,
                    Crankcase_Heater_Power_per_Compressor=15,
                    Number_of_Compressors=2,
                    Ratio_of_Compressor_Size_to_Total_Compressor_Capacity=0.5,
                    Maximum_Outdoor_DryBulb_Temperature_for_Crankcase_Heater=5,
                    Defrost_Strategy='Resistive',
                    Defrost_Control='Timed',
                    Defrost_Energy_Input_Ratio_Modifier_Function_of_Temperature_Curve_Name='',
                    Defrost_Time_Period_Fraction=0,
                    Resistive_Defrost_Heater_Capacity='autosize',
                    Maximum_Outdoor_Drybulb_Temperature_for_Defrost_Operation=5,
                    Condenser_Type='AirCooled',
                    Condenser_Inlet_Node_Name='VRF Outdoor Unit_'+
                                              zn+
                                              ' Outdoor Air Node',
                    Condenser_Outlet_Node_Name='',
                    Water_Condenser_Volume_Flow_Rate='autosize',
                    Evaporative_Condenser_Effectiveness=0.9,
                    Evaporative_Condenser_Air_Flow_Rate='autosize',
                    Evaporative_Condenser_Pump_Rated_Power_Consumption='autosize',
                    Supply_Water_Storage_Tank_Name='',
                    Basin_Heater_Capacity=0,
                    Basin_Heater_Setpoint_Temperature=2,
                    Basin_Heater_Operating_Schedule_Name='On 24/7',
                    Fuel_Type='Electricity',
                    Minimum_Condenser_Inlet_Node_Temperature_in_Heat_Recovery_Mode=-10,
                    Maximum_Condenser_Inlet_Node_Temperature_in_Heat_Recovery_Mode=40,
                    Heat_Recovery_Cooling_Capacity_Modifier_Curve_Name=
                    'VRF Heat Recovery Cooling Capacity Modifier',
                    Initial_Heat_Recovery_Cooling_Capacity_Fraction=0.5,
                    Heat_Recovery_Cooling_Capacity_Time_Constant=0.15,
                    Heat_Recovery_Cooling_Energy_Modifier_Curve_Name=
                    'VRF Heat Recovery Cooling Energy Modifier',
                    Initial_Heat_Recovery_Cooling_Energy_Fraction=1,
                    Heat_Recovery_Cooling_Energy_Time_Constant=0,
                    Heat_Recovery_Heating_Capacity_Modifier_Curve_Name=
                    'VRF Heat Recovery Heating Capacity Modifier',
                    Initial_Heat_Recovery_Heating_Capacity_Fraction=1,
                    Heat_Recovery_Heating_Capacity_Time_Constant=0.15,
                    Heat_Recovery_Heating_Energy_Modifier_Curve_Name=
                    'VRF Heat Recovery Heating Energy Modifier',
                    Initial_Heat_Recovery_Heating_Energy_Fraction=1,
                    Heat_Recovery_Heating_Energy_Time_Constant=0
                    )
            else:
                self.idf1.newidfobject(
                    'AirConditioner:VariableRefrigerantFlow',
                    Heat_Pump_Name='VRF Outdoor Unit_' + zn,
                    Availability_Schedule_Name='On 24/7',
                    Gross_Rated_Total_Cooling_Capacity='autosize',
                    Gross_Rated_Cooling_COP=2,
                    # Minimum_Condenser_Inlet_Node_Temperature_in_Cooling_Mode=-6,
                    # Maximum_Condenser_Inlet_Node_Temperature_in_Cooling_Mode=43,
                    Minimum_Outdoor_Temperature_in_Cooling_Mode=-6,
                    Maximum_Outdoor_Temperature_in_Cooling_Mode=43,
                    Cooling_Capacity_Ratio_Modifier_Function_of_Low_Temperature_Curve_Name=
                    'VRFCoolCapFT',
                    Cooling_Capacity_Ratio_Boundary_Curve_Name='VRFCoolCapFTBoundary',
                    Cooling_Capacity_Ratio_Modifier_Function_of_High_Temperature_Curve_Name=
                    'VRFCoolCapFTHi',
                    Cooling_Energy_Input_Ratio_Modifier_Function_of_Low_Temperature_Curve_Name=
                    'VRFCoolEIRFT',
                    Cooling_Energy_Input_Ratio_Boundary_Curve_Name='VRFCoolEIRFTBoundary',
                    Cooling_Energy_Input_Ratio_Modifier_Function_of_High_Temperature_Curve_Name=
                    'VRFCoolEIRFTHi',
                    Cooling_Energy_Input_Ratio_Modifier_Function_of_Low_PartLoad_Ratio_Curve_Name=
                    'CoolingEIRLowPLR',
                    Cooling_Energy_Input_Ratio_Modifier_Function_of_High_PartLoad_Ratio_Curve_Name=
                    'CoolingEIRHiPLR',
                    Cooling_Combination_Ratio_Correction_Factor_Curve_Name=
                    'CoolingCombRatio',
                    Cooling_PartLoad_Fraction_Correlation_Curve_Name='VRFCPLFFPLR',
                    Gross_Rated_Heating_Capacity='autosize',
                    Rated_Heating_Capacity_Sizing_Ratio=1,
                    Gross_Rated_Heating_COP=2.1,
                    Minimum_Outdoor_Temperature_in_Heating_Mode=-20,
                    Maximum_Outdoor_Temperature_in_Heating_Mode=40,
                    Heating_Capacity_Ratio_Modifier_Function_of_Low_Temperature_Curve_Name=
                    'VRFHeatCapFT',
                    Heating_Capacity_Ratio_Boundary_Curve_Name='VRFHeatCapFTBoundary',
                    Heating_Capacity_Ratio_Modifier_Function_of_High_Temperature_Curve_Name=
                    'VRFHeatCapFTHi',
                    Heating_Energy_Input_Ratio_Modifier_Function_of_Low_Temperature_Curve_Name=
                    'VRFHeatEIRFT',
                    Heating_Energy_Input_Ratio_Boundary_Curve_Name='VRFHeatEIRFTBoundary',
                    Heating_Energy_Input_Ratio_Modifier_Function_of_High_Temperature_Curve_Name=
                    'VRFHeatEIRFTHi',
                    Heating_Performance_Curve_Outdoor_Temperature_Type='WetBulbTemperature',
                    Heating_Energy_Input_Ratio_Modifier_Function_of_Low_PartLoad_Ratio_Curve_Name=
                    'HeatingEIRLowPLR',
                    Heating_Energy_Input_Ratio_Modifier_Function_of_High_PartLoad_Ratio_Curve_Name=
                    'HeatingEIRHiPLR',
                    Heating_Combination_Ratio_Correction_Factor_Curve_Name=
                    'HeatingCombRatio',
                    Heating_PartLoad_Fraction_Correlation_Curve_Name='VRFCPLFFPLR',
                    Minimum_Heat_Pump_PartLoad_Ratio=0.2,
                    Zone_Name_for_Master_Thermostat_Location='',
                    Master_Thermostat_Priority_Control_Type='LoadPriority',
                    Thermostat_Priority_Schedule_Name='',
                    Zone_Terminal_Unit_List_Name='VRF Outdoor Unit_' + zn + ' Zone List',
                    Heat_Pump_Waste_Heat_Recovery='Yes',
                    Equivalent_Piping_Length_used_for_Piping_Correction_Factor_in_Cooling_Mode=50,
                    Vertical_Height_used_for_Piping_Correction_Factor=15,
                    Piping_Correction_Factor_for_Length_in_Cooling_Mode_Curve_Name=
                    'CoolingLengthCorrectionFactor',
                    Piping_Correction_Factor_for_Height_in_Cooling_Mode_Coefficient=0,
                    Equivalent_Piping_Length_used_for_Piping_Correction_Factor_in_Heating_Mode=50,
                    Piping_Correction_Factor_for_Length_in_Heating_Mode_Curve_Name=
                    'VRF Piping Correction Factor for Length in Heating Mode',
                    Piping_Correction_Factor_for_Height_in_Heating_Mode_Coefficient=0,
                    Crankcase_Heater_Power_per_Compressor=15,
                    Number_of_Compressors=2,
                    Ratio_of_Compressor_Size_to_Total_Compressor_Capacity=0.5,
                    Maximum_Outdoor_DryBulb_Temperature_for_Crankcase_Heater=5,
                    Defrost_Strategy='Resistive',
                    Defrost_Control='Timed',
                    Defrost_Energy_Input_Ratio_Modifier_Function_of_Temperature_Curve_Name='',
                    Defrost_Time_Period_Fraction=0,
                    Resistive_Defrost_Heater_Capacity='autosize',
                    Maximum_Outdoor_Drybulb_Temperature_for_Defrost_Operation=5,
                    Condenser_Type='AirCooled',
                    Condenser_Inlet_Node_Name='VRF Outdoor Unit_' +
                                              zn +
                                              ' Outdoor Air Node',
                    Condenser_Outlet_Node_Name='',
                    Water_Condenser_Volume_Flow_Rate='autosize',
                    Evaporative_Condenser_Effectiveness=0.9,
                    Evaporative_Condenser_Air_Flow_Rate='autosize',
                    Evaporative_Condenser_Pump_Rated_Power_Consumption='autosize',
                    Supply_Water_Storage_Tank_Name='',
                    Basin_Heater_Capacity=0,
                    Basin_Heater_Setpoint_Temperature=2,
                    Basin_Heater_Operating_Schedule_Name='On 24/7',
                    Fuel_Type='Electricity',
                    Minimum_Outdoor_Temperature_in_Heat_Recovery_Mode=-10,
                    Maximum_Outdoor_Temperature_in_Heat_Recovery_Mode=40,
                    Heat_Recovery_Cooling_Capacity_Modifier_Curve_Name=
                    'VRF Heat Recovery Cooling Capacity Modifier',
                    Initial_Heat_Recovery_Cooling_Capacity_Fraction=0.5,
                    Heat_Recovery_Cooling_Capacity_Time_Constant=0.15,
                    Heat_Recovery_Cooling_Energy_Modifier_Curve_Name=
                    'VRF Heat Recovery Cooling Energy Modifier',
                    Initial_Heat_Recovery_Cooling_Energy_Fraction=1,
                    Heat_Recovery_Cooling_Energy_Time_Constant=0,
                    Heat_Recovery_Heating_Capacity_Modifier_Curve_Name=
                    'VRF Heat Recovery Heating Capacity Modifier',
                    Initial_Heat_Recovery_Heating_Capacity_Fraction=1,
                    Heat_Recovery_Heating_Capacity_Time_Constant=0.15,
                    Heat_Recovery_Heating_Energy_Modifier_Curve_Name=
                    'VRF Heat Recovery Heating Energy Modifier',
                    Initial_Heat_Recovery_Heating_Energy_Fraction=1,
                    Heat_Recovery_Heating_Energy_Time_Constant=0
                )

            if verboseMode:
                print('VRF Outdoor Unit_'+
                      zn+
                      ' AirConditioner:VariableRefrigerantFlow '
                      'Object has been added')

    OANDlist = [i for i in self.idf1.idfobjects['OutdoorAir:NodeList']]
    # print(OANDlist)

    for i in range(len(OANDlist)):
        firstOAND = self.idf1.idfobjects['OutdoorAir:NodeList'][-1]
        self.idf1.removeidfobject(firstOAND)

    del OANDlist

    ZTULlist = [i for i in self.idf1.idfobjects['ZoneTerminalUnitList']]
    # print(ZTULlist)

    for i in range(len(ZTULlist)):
        firstZTUL = self.idf1.idfobjects['ZoneTerminalUnitList'][-1]
        self.idf1.removeidfobject(firstZTUL)

    del ZTULlist

    for zn in self.zonenames_orig:
        self.idf1.newidfobject(
            'OutdoorAir:NodeList',
            Node_or_NodeList_Name_1='VRF Outdoor Unit_'+zn+' Outdoor Air Node'
            )
        if verboseMode:
            print('VRF Outdoor Unit_'+
                  zn+
                  ' Outdoor Air Node Object has been added')

        self.idf1.newidfobject(
            'ZoneTerminalUnitList',
            Zone_Terminal_Unit_List_Name='VRF Outdoor Unit_'+zn+' Zone List',
            Zone_Terminal_Unit_Name_1=zn+' VRF Indoor Unit'
            )
        if verboseMode:
            print('VRF Outdoor Unit_'+zn+' Zone List Object has been added')

    ZCTlist = [i for i in self.idf1.idfobjects['ZoneControl:Thermostat']]
    # print(ZCTlist)

    for i in range(len(ZCTlist)):
        firstZCT = self.idf1.idfobjects['ZoneControl:Thermostat'][-1]
        self.idf1.removeidfobject(firstZCT)

    del ZCTlist

    for zn in self.zonenames_orig:
        self.idf1.newidfobject(
            'ZoneControl:Thermostat',
            Name=zn+' Thermostat',
            Zone_or_ZoneList_Name=zn,
            Control_Type_Schedule_Name='Control type schedule: Always 4',
            Control_1_Object_Type='ThermostatSetpoint:DualSetpoint',
            Control_1_Name=zn+' Dual SP'
            )

    sizingzonelist = [i for i in self.idf1.idfobjects['Sizing:Zone']]
    # print(sizingzonelist)

    for i in range(len(sizingzonelist)):
        firstsizingzone = self.idf1.idfobjects['Sizing:Zone'][-1]
        self.idf1.removeidfobject(firstsizingzone)

    del sizingzonelist

    if TempCtrl == 'pmv':
        supply_air_temp_input_method = 'SupplyAirTemperature'
    elif TempCtrl.lower() == 'temp' or TempCtrl.lower() == 'temperature':
        supply_air_temp_input_method = 'TemperatureDifference'

    for zn in self.zonenames_orig:
        self.idf1.newidfobject(
            'Sizing:Zone',
            Zone_or_ZoneList_Name=zn,
            Zone_Cooling_Design_Supply_Air_Temperature_Input_Method=
            supply_air_temp_input_method,
            Zone_Cooling_Design_Supply_Air_Temperature=14,
            Zone_Cooling_Design_Supply_Air_Temperature_Difference=5,
            Zone_Heating_Design_Supply_Air_Temperature_Input_Method=
            supply_air_temp_input_method,
            Zone_Heating_Design_Supply_Air_Temperature=50,
            Zone_Heating_Design_Supply_Air_Temperature_Difference=5,
            Zone_Cooling_Design_Supply_Air_Humidity_Ratio=0.009,
            Zone_Heating_Design_Supply_Air_Humidity_Ratio=0.004,
            Design_Specification_Outdoor_Air_Object_Name=
            zn+' Design Specification Outdoor Air Object',
            Zone_Heating_Sizing_Factor=1.25,
            Zone_Cooling_Sizing_Factor=1.15,
            Cooling_Design_Air_Flow_Method='DesignDay',
            Cooling_Design_Air_Flow_Rate=0,
            Cooling_Minimum_Air_Flow_per_Zone_Floor_Area=0.00076,
            Cooling_Minimum_Air_Flow=0,
            Cooling_Minimum_Air_Flow_Fraction=0,
            Heating_Design_Air_Flow_Method='DesignDay',
            Heating_Design_Air_Flow_Rate=0,
            Heating_Maximum_Air_Flow_per_Zone_Floor_Area=0.00203,
            Heating_Maximum_Air_Flow=0.14158,
            Heating_Maximum_Air_Flow_Fraction=0.3,
            Design_Specification_Zone_Air_Distribution_Object_Name=
            zn+' Design Specification Zone Air Distribution Object',
            Account_for_Dedicated_Outdoor_Air_System='Yes',
            Dedicated_Outdoor_Air_System_Control_Strategy='NeutralSupplyAir',
            Dedicated_Outdoor_Air_Low_Setpoint_Temperature_for_Design='autosize',
            Dedicated_Outdoor_Air_High_Setpoint_Temperature_for_Design='autosize'
            )
        if verboseMode:
            print(zn+' Sizing:Zone Object has been added')

    DSOAlist = [i
                for i
                in self.idf1.idfobjects['DesignSpecification:OutdoorAir']]
    # print(DSOAlist)

    for i in range(len(DSOAlist)):
        firstDSOA = self.idf1.idfobjects['DesignSpecification:OutdoorAir'][-1]
        self.idf1.removeidfobject(firstDSOA)

    del DSOAlist

    for zn in self.zonenames_orig:
        self.idf1.newidfobject(
            'DesignSpecification:OutdoorAir',
            Name=zn+' Design Specification Outdoor Air Object',
            Outdoor_Air_Method='Flow/Person',
            Outdoor_Air_Flow_per_Person=0.00944,
            Outdoor_Air_Flow_per_Zone_Floor_Area=0,
            Outdoor_Air_Flow_per_Zone=0,
            Outdoor_Air_Flow_Air_Changes_per_Hour=0,
            Outdoor_Air_Schedule_Name='On 24/7'
            )
        if verboseMode:
            print(zn+' Design Specification Outdoor Air Object has been added')

    DSZADlist = [i
                 for i
                 in self.idf1.idfobjects
                 ['DesignSpecification:ZoneAirDistribution']]
    # print(DSZADlist)

    for i in range(len(DSZADlist)):
        firstDSZAD = self.idf1.idfobjects[
            'DesignSpecification:ZoneAirDistribution'
        ][-1]
        self.idf1.removeidfobject(firstDSZAD)

    del DSZADlist

    for zn in self.zonenames_orig:
        self.idf1.newidfobject(
            'DesignSpecification:ZoneAirDistribution',
            Name=zn+' Design Specification Zone Air Distribution Object',
            Zone_Air_Distribution_Effectiveness_in_Cooling_Mode=1,
            Zone_Air_Distribution_Effectiveness_in_Heating_Mode=1,
            Zone_Air_Distribution_Effectiveness_Schedule_Name='',
            Zone_Secondary_Recirculation_Fraction=0
            )
        if verboseMode:
            print(zn+' Design Specification Zone Air Distribution '
                     'Object has been added')

    nodelistlist = [i for i in self.idf1.idfobjects['NodeList']]
    # print(nodelistlist)

    for i in range(len(nodelistlist)):
        firstnodelist = self.idf1.idfobjects['NodeList'][-1]
        self.idf1.removeidfobject(firstnodelist)

    del nodelistlist

    for zn in self.zonenames_orig:
        self.idf1.newidfobject(
            'NodeList',
            Name=zn+' Air Inlet Node List',
            Node_1_Name=zn+' VRF Indoor Unit Supply Outlet'
            )
        self.idf1.newidfobject(
            'NodeList',
            Name=zn+' Air Exhaust Node List',
            Node_1_Name=zn+' VRF Indoor Unit Return'
            )
        if verboseMode:
            print(zn+' Nodelist Objects has been added')

    ZHEClist = [i
                for i
                in self.idf1.idfobjects['ZoneHVAC:EquipmentConnections']]
    # print(ZHEClist)

    for i in range(len(ZHEClist)):
        firstZHEC = self.idf1.idfobjects['ZoneHVAC:EquipmentConnections'][-1]
        self.idf1.removeidfobject(firstZHEC)

    del ZHEClist

    for zn in self.zonenames_orig:
        self.idf1.newidfobject(
            'ZoneHVAC:EquipmentConnections',
            Zone_Name=zn,
            Zone_Conditioning_Equipment_List_Name=zn+' Equipment',
            Zone_Air_Inlet_Node_or_NodeList_Name=zn+' Air Inlet Node List',
            Zone_Air_Exhaust_Node_or_NodeList_Name=zn+' Air Exhaust Node List',
            Zone_Air_Node_Name=zn+' Zone Air Node',
            Zone_Return_Air_Node_or_NodeList_Name=zn+' Return Outlet'
            )
        if verboseMode:
            print(zn+' ZoneHVAC:EquipmentConnections Objects has been added')

    ZHELlist = [i for i in self.idf1.idfobjects['ZoneHVAC:EquipmentList']]
    # print(ZHELlist)

    for i in range(len(ZHELlist)):
        firstZHEL = self.idf1.idfobjects['ZoneHVAC:EquipmentList'][-1]
        self.idf1.removeidfobject(firstZHEL)

    del ZHELlist

    for zn in self.zonenames_orig:
        self.idf1.newidfobject(
            'ZoneHVAC:EquipmentList',
            defaultvalues=False,
            Name=zn+' Equipment',
            Load_Distribution_Scheme='SequentialLoad',
            Zone_Equipment_1_Object_Type=
            'ZoneHVAC:TerminalUnit:VariableRefrigerantFlow',
            Zone_Equipment_1_Name=zn+' VRF Indoor Unit',
            Zone_Equipment_1_Cooling_Sequence=1,
            Zone_Equipment_1_Heating_or_NoLoad_Sequence=1,
            Zone_Equipment_1_Sequential_Cooling_Fraction_Schedule_Name='',
            Zone_Equipment_1_Sequential_Heating_Fraction_Schedule_Name=''
            )
        if verboseMode:
            print(zn+' ZoneHVAC:EquipmentList Objects has been added')

    ZHTUVRFlist = [i
                   for i
                   in self.idf1.idfobjects
                   ['ZoneHVAC:TerminalUnit:VariableRefrigerantFlow']]
    # print(ZHTUVRFlist)

    for i in range(len(ZHTUVRFlist)):
        firstZHTUVRF = self.idf1.idfobjects[
            'ZoneHVAC:TerminalUnit:VariableRefrigerantFlow'
        ][-1]
        self.idf1.removeidfobject(firstZHTUVRF)

    del ZHTUVRFlist

    for zn in self.zonenames_orig:
        self.idf1.newidfobject(
            'ZoneHVAC:TerminalUnit:VariableRefrigerantFlow',
            Zone_Terminal_Unit_Name=zn+' VRF Indoor Unit',
            Terminal_Unit_Availability_Schedule='On 24/7',
            Terminal_Unit_Air_Inlet_Node_Name=zn+' VRF Indoor Unit Return',
            Terminal_Unit_Air_Outlet_Node_Name=
            zn+' VRF Indoor Unit Supply Outlet',
            Cooling_Supply_Air_Flow_Rate='autosize',
            No_Cooling_Supply_Air_Flow_Rate='autosize',
            Heating_Supply_Air_Flow_Rate='autosize',
            No_Heating_Supply_Air_Flow_Rate='autosize',
            Cooling_Outdoor_Air_Flow_Rate=0,
            Heating_Outdoor_Air_Flow_Rate=0,
            No_Load_Outdoor_Air_Flow_Rate=0,
            Supply_Air_Fan_Operating_Mode_Schedule_Name='On 24/7',
            Supply_Air_Fan_Placement='DrawThrough',
            Supply_Air_Fan_Object_Type='Fan:ConstantVolume',
            Supply_Air_Fan_Object_Name=zn+' VRF Indoor Unit Supply Fan',
            Outside_Air_Mixer_Object_Type='',
            Outside_Air_Mixer_Object_Name='',
            Cooling_Coil_Object_Type='Coil:Cooling:DX:VariableRefrigerantFlow',
            Cooling_Coil_Object_Name=zn+' VRF Indoor Unit DX Cooling Coil',
            Heating_Coil_Object_Type='Coil:Heating:DX:VariableRefrigerantFlow',
            Heating_Coil_Object_Name=zn+' VRF Indoor Unit DX Heating Coil',
            Zone_Terminal_Unit_On_Parasitic_Electric_Energy_Use=30,
            Zone_Terminal_Unit_Off_Parasitic_Electric_Energy_Use=20,
            Rated_Heating_Capacity_Sizing_Ratio='',
            Availability_Manager_List_Name=''
            )
        if verboseMode:
            print(zn+' ZoneHVAC:TerminalUnit:VariableRefrigerantFlow '
                     'Object has been added')

    CCDXVRFlist = [i
                   for i
                   in self.idf1.idfobjects
                   ['Coil:Cooling:DX:VariableRefrigerantFlow']]
    # print(CCDXVRFlist)

    for i in range(len(CCDXVRFlist)):
        firstCCDCVRF = self.idf1.idfobjects[
            'Coil:Cooling:DX:VariableRefrigerantFlow'
        ][-1]
        self.idf1.removeidfobject(firstCCDCVRF)

    del CCDXVRFlist

    for zn in self.zonenames_orig:
        self.idf1.newidfobject(
            'Coil:Cooling:DX:VariableRefrigerantFlow',
            Name=zn+' VRF Indoor Unit DX Cooling Coil',
            Availability_Schedule_Name='On 24/7',
            Gross_Rated_Total_Cooling_Capacity='autosize',
            Gross_Rated_Sensible_Heat_Ratio='autosize',
            Rated_Air_Flow_Rate='autosize',
            Cooling_Capacity_Ratio_Modifier_Function_of_Temperature_Curve_Name=
            'VRFTUCoolCapFT',
            Cooling_Capacity_Modifier_Curve_Function_of_Flow_Fraction_Name=
            'VRFACCoolCapFFF',
            Coil_Air_Inlet_Node=zn+' VRF Indoor Unit Return',
            Coil_Air_Outlet_Node=zn+' VRF Indoor Unit DX Cooling Coil Outlet',
            Name_of_Water_Storage_Tank_for_Condensate_Collection=''
            )
        if verboseMode:
            print(zn+' Coil:Cooling:DX:VariableRefrigerantFlow '
                     'Object has been added')

    CHDXVRFlist = [i
                   for i
                   in self.idf1.idfobjects
                   ['Coil:Heating:DX:VariableRefrigerantFlow']]
    # print(CHDXVRFlist)

    for i in range(len(CHDXVRFlist)):
        firstCHDXVRF = self.idf1.idfobjects[
            'Coil:Heating:DX:VariableRefrigerantFlow'
        ][-1]
        self.idf1.removeidfobject(firstCHDXVRF)

    del CHDXVRFlist

    for zn in self.zonenames_orig:
        self.idf1.newidfobject(
            'Coil:Heating:DX:VariableRefrigerantFlow',
            Name=zn+' VRF Indoor Unit DX Heating Coil',
            Availability_Schedule='On 24/7',
            Gross_Rated_Heating_Capacity='autosize',
            Rated_Air_Flow_Rate='autosize',
            Coil_Air_Inlet_Node=zn+' VRF Indoor Unit DX Cooling Coil Outlet',
            Coil_Air_Outlet_Node=zn+' VRF Indoor Unit DX Heating Coil Outlet',
            Heating_Capacity_Ratio_Modifier_Function_of_Temperature_Curve_Name=
            'VRFTUHeatCapFT',
            Heating_Capacity_Modifier_Function_of_Flow_Fraction_Curve_Name=
            'VRFACCoolCapFFF'
            )
        if verboseMode:
            print(zn+' Coil:Heating:DX:VariableRefrigerantFlow '
                     'Object has been added')

    FCVlist = [i for i in self.idf1.idfobjects['Fan:ConstantVolume']]
    # print(FCVlist)

    for i in range(len(FCVlist)):
        firstFCV = self.idf1.idfobjects['Fan:ConstantVolume'][-1]
        self.idf1.removeidfobject(firstFCV)

    del FCVlist

    for zn in self.zonenames_orig:
        self.idf1.newidfobject(
            'Fan:ConstantVolume',
            Name=zn+' VRF Indoor Unit Supply Fan',
            Availability_Schedule_Name='On 24/7',
            Fan_Total_Efficiency=0.7,
            Pressure_Rise=100,
            Maximum_Flow_Rate='autosize',
            Motor_Efficiency=0.9,
            Motor_In_Airstream_Fraction=1,
            Air_Inlet_Node_Name=zn+' VRF Indoor Unit DX Heating Coil Outlet',
            Air_Outlet_Node_Name=zn+' VRF Indoor Unit Supply Outlet',
            EndUse_Subcategory='General'
            )
        if verboseMode:
            print(zn+' Fan:ConstantVolume Object has been added')


def addForscriptSchVRFsystem(self, verboseMode: bool = True):
    """Add FORSCRIPT Schedules for each zone for VRFsystem."""
    for zn in self.zonenames:
        if "AHST_Sch_"+zn in [sch.Name
                                    for sch
                                    in self.idf1.idfobjects['Schedule:Compact']]:
            if verboseMode:
                print('AHST_Sch_'+zn+' Schedule already was in the model')
        else:
            self.idf1.newidfobject(
                'Schedule:Compact',
                Name="AHST_Sch_"+zn,
                Schedule_Type_Limits_Name="Any Number",
                Field_1='Through: 12/31',
                Field_2='For: AllDays',
                Field_3='Until: 24:00,20'
                )
            if verboseMode:
                print('AHST_Sch_'+zn+' Schedule has been added')

        if "ACST_Sch_"+zn in [sch.Name
                                    for sch
                                    in self.idf1.idfobjects['Schedule:Compact']]:
            if verboseMode:
                print('ACST_Sch_'+zn+' Schedule already was in the model')
        else:
            self.idf1.newidfobject(
                'Schedule:Compact',
                Name="ACST_Sch_"+zn,
                Schedule_Type_Limits_Name="Any Number",
                Field_1='Through: 12/31',
                Field_2='For: AllDays',
                Field_3='Until: 24:00,24'
                )
            if verboseMode:
                print('ACST_Sch_'+zn+' Schedule has been added')

    # allschedules=([i for i in self.idf1.idfobjects['Schedule:Compact']])
    # print(allschedules)

    TSDSlist = ([i
                 for i
                 in self.idf1.idfobjects['ThermostatSetpoint:DualSetpoint']])
    # print(TSDSlist)

    for i in range(len(TSDSlist)):
        firstTSDS = self.idf1.idfobjects['ThermostatSetpoint:DualSetpoint'][-1]
        self.idf1.removeidfobject(firstTSDS)

    del TSDSlist

    for i in range(len(self.zonenames_orig)):
        self.idf1.newidfobject(
            'ThermostatSetpoint:DualSetpoint',
            Name=self.zonenames_orig[i]+' Dual SP',
            Heating_Setpoint_Temperature_Schedule_Name=
            "AHST_Sch_"+self.zonenames[i],
            Cooling_Setpoint_Temperature_Schedule_Name=
            "ACST_Sch_"+self.zonenames[i]
            )


def checkVentIsOn(self, verboseMode: bool = True):
    """Check ventilation settings."""
    if "Vent_SP_temp" in [sch.Name
                          for sch
                          in self.idf1.idfobjects['Schedule:Compact']]:
        if verboseMode:
            print("Vent_SP_temp Schedule already was in the model")
    else:
        self.idf1.newidfobject(
            'Schedule:Compact',
            Name="Vent_SP_temp",
            Schedule_Type_Limits_Name="Any Number",
            Field_1='Through: 12/31',
            Field_2='For: AllDays',
            Field_3='Until: 24:00,24'
            )
        if verboseMode:
            print("Vent_SP_temp Schedule has been added")

    Wlist = ([w
              for w
              in self.idf1.idfobjects
              ['AirflowNetwork:MultiZone:Component:DetailedOpening']
              if w.Name.endswith('_Win')])
    # print(Wlist)
    for w in Wlist:
        w.Height_Factor_for_Opening_Factor_1 = 1
        w.Start_Height_Factor_for_Opening_Factor_1 = 0
        w.Width_Factor_for_Opening_Factor_2 = 1
        w.Height_Factor_for_Opening_Factor_2 = 1
        w.Start_Height_Factor_for_Opening_Factor_2 = 0

    Wlist_2 = ([w
                for w
                in self.idf1.idfobjects
                ['AirflowNetwork:MultiZone:Surface']
                if w.Surface_Name.endswith('_Win')])
    # print(Wlist_2)
    for w in Wlist_2:
        w.Ventilation_Control_Mode = 'Temperature'
        w.Ventilation_Control_Zone_Temperature_Setpoint_Schedule_Name = 'Vent_SP_temp'
        w.Venting_Availability_Schedule_Name = 'On'

    # Checking if all internal doors are always opened

    Dlist = ([d
              for d
              in self.idf1.idfobjects
              ['AirflowNetwork:MultiZone:Component:DetailedOpening']
              if d.Name.endswith('_Door')])
    for d in Dlist:
        d.Width_Factor_for_Opening_Factor_2 = 1

    Dlist_2 = ([d
                for d
                in self.idf1.idfobjects
                ['AirflowNetwork:MultiZone:Surface']
                if d.Surface_Name.endswith('_Door')])
    for d in Dlist_2:
        d.Venting_Availability_Schedule_Name = 'On'




from accim.sim import accis
accis.addAccis(ScriptType='ex_ac',
               Output_keep_existing=False,
               Output_type='standard',
               Output_freqs=['hourly'],
               EnergyPlus_version='9.5',
               TempCtrl='temp',
               ComfStand=[1],
               CAT=[2],
               ComfMod=[3],
               confirmGen=True)


from eppy import modeleditor
from eppy.modeleditor import IDF
import os

iddfile = r"C:\EnergyPlusV9-4-0\Energy+.idd"
# fname1 = r"D:\OneDrive - UNIVERSIDAD DE SEVILLA\Papers OneDrive\VPO_cadiz_parametrico\BA_V01.idf"

path = r'C:\Users\daniel.sanchez\Documents\Personal\VPO_parametrico\input_IDFs'
outputpath = r'C:\Users\daniel.sanchez\Documents\Personal\VPO_parametrico\output_IDFs'

filelist = [file.split('.idf')[0] for file in os.listdir(path) if file.endswith('.idf')]
print(filelist)

IDF.setiddname(iddfile)

# idf1.printidf()



facade_cond_list = [
    0.01292,
    0.02673,
    0.04152,
    0.05741,
    0.07453,
    0.09302,
    0.11305,
    0.13482,
    0.15858,
    0.18460,
    0.21323,
    0.24487,
    0.28003,
    0.31934,
    0.36357,
    0.41371,
    0.47103,
    0.53718,
    0.61438,
    0.70565,
]

facade_cond_list_full = [
    0.01292,
    0.02673,
    0.04152,
    0.05741,
    0.07453,
    0.09302,
    0.11305,
    0.13482,
    0.15858,
    0.18460,
    0.21323,
    0.24487,
    0.28003,
    0.31934,
    0.36357,
    0.41371,
    0.47103,
    0.53718,
    0.61438,
    0.70565,
    0.81522,
    0.94921,
    1.11681,
    1.33249,
    1.62037,
    2.02402,
    2.63085,
    3.64583,
    5.68946,
    11.93182
]

roof_cond_list = [
    0.03059,
    0.06240,
    0.09552,
    0.13001,
    0.16598,
    0.20350,
    0.24270,
    0.28369,
    0.32658,
    0.37152,
    0.41865,
    0.46814,
    0.52017,
    0.57495,
    0.63269,
    0.69364,
    0.75808,
    0.82632,
    0.89870,
    0.97561,
]

roof_cond_list_full = [
    0.03059,
    0.06240,
    0.09552,
    0.13001,
    0.16598,
    0.20350,
    0.24270,
    0.28369,
    0.32658,
    0.37152,
    0.41865,
    0.46814,
    0.52017,
    0.57495,
    0.63269,
    0.69364,
    0.75808,
    0.82632,
    0.89870,
    0.97561,
    1.05749,
    1.14484,
    1.23822,
    1.33829,
    1.44578,
    1.56156,
    1.68662,
    1.82213,
    1.96944,
    2.13018
]

floor_cond_list_full = [
    0.01679,
    0.03531,
    0.05586,
    0.07879,
    0.10452,
    0.13362,
    0.16678,
    0.20493,
    0.24927,
    0.30145,
    0.36375,
    0.43943,
    0.53333,
    0.65291,
    0.81038,
    1.02714,
    1.34445,
    1.85339,
    2.80267,
    5.19941,
    22.97796,
    -10.89783,
    -4.64513,
    -3.04410,
    -2.31123,
    -1.89099,
    -1.61850,
    -1.42750,
    -1.28618,
    -1.17739
]

floor_cond_list = [
    0.01679,
    0.03531,
    0.05586,
    0.07879,
    0.10452,
    0.13362,
    0.16678,
    0.20493,
    0.24927,
    0.30145,
    0.36375,
    0.43943,
    0.53333,
    0.65291,
    0.81038,
    1.02714,
    1.34445,
    1.85339,
    2.80267,
    5.19941,
]

for file in filelist:
    filename = file
    print(file)
    fname1 = path+'/'+filename + '.idf'
    idf1 = IDF(fname1)

    if 'Cast Concrete_.1' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
        mat_cast_concrete = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'Cast Concrete_.1'][0]
        mat_cast_concrete.Roughness = 'Rough'
        mat_cast_concrete.Thickness = 0.1
        mat_cast_concrete.Conductivity = 1.13
        mat_cast_concrete.Density = 2000
        mat_cast_concrete.Specific_Heat = 1000
        mat_cast_concrete.Thermal_Absorptance = 0.9
        mat_cast_concrete.Solar_Absorptance = 0.6
        mat_cast_concrete.Visible_Absorptance = 0.6
    else:
        idf1.newidfobject('MATERIAL',
                          Name='Cast Concrete_.1',
                          Roughness='Rough',
                          Thickness=0.1,
                          Conductivity=1.13,
                          Density=2000,
                          Specific_Heat=1000,
                          Thermal_Absorptance=0.9,
                          Solar_Absorptance=0.6,
                          Visible_Absorptance=0.6
                          )

    if 'O SO1_.16' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
        mat_S01 = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'O SO1_.16'][0]
        mat_S01.Roughness = 'Rough'
        mat_S01.Thickness = 0.16
        mat_S01.Conductivity = 0.01679
        mat_S01.Density = 1700
        mat_S01.Specific_Heat = 1000
        mat_S01.Thermal_Absorptance = 0.9
        mat_S01.Solar_Absorptance = 0.7
        mat_S01.Visible_Absorptance = 0.7
    else:
        idf1.newidfobject('MATERIAL',
                          Name='O SO1_.16',
                          Roughness='Rough',
                          Thickness=0.16,
                          Conductivity=0.01679,
                          Density=1700,
                          Specific_Heat=1000,
                          Thermal_Absorptance=0.9,
                          Solar_Absorptance=0.7,
                          Visible_Absorptance=0.7
                          )

    if 'Floor/Roof Screed_.O7' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
        mat_screed = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'Floor/Roof Screed_.O7'][0]
        mat_screed.Roughness = 'Rough'
        mat_screed.Thickness = 0.07
        mat_screed.Conductivity = 0.41
        mat_screed.Density = 1200
        mat_screed.Specific_Heat = 840
        mat_screed.Thermal_Absorptance = 0.9
        mat_screed.Solar_Absorptance = 0.73
        mat_screed.Visible_Absorptance = 0.73
    else:
        idf1.newidfobject('MATERIAL',
                          Name='Floor/Roof Screed_.O7',
                          Roughness='Rough',
                          Thickness=0.07,
                          Conductivity=0.41,
                          Density=1200,
                          Specific_Heat=840,
                          Thermal_Absorptance=0.9,
                          Solar_Absorptance=0.73,
                          Visible_Absorptance=0.73
                          )

    if 'OO SO1' in [i.Name for i in idf1.idfobjects['CONSTRUCTION']]:
        cons_S01 = [i for i in idf1.idfobjects['CONSTRUCTION'] if i.Name == 'OO SO1'][0]
        cons_S01.Name = 'OO SO1'
        cons_S01.Outside_Layer = 'Cast Concrete_.1'
        cons_S01.Layer_2 = 'O SO1_.16'
        cons_S01.Layer_3 = 'Floor/Roof Screed_.O7'
    else:
        idf1.newidfobject(
            'CONSTRUCTION',
            Name='OO SO1',
            Outside_Layer='Cast Concrete_.1',
            Layer_2='O SO1_.16',
            Layer_3='Floor/Roof Screed_.O7'
        )

    if 'OO SO1_Rev' in [i.Name for i in idf1.idfobjects['CONSTRUCTION']]:
        cons_S01_rev = [i for i in idf1.idfobjects['CONSTRUCTION'] if i.Name == 'OO SO1_Rev'][0]
        cons_S01_rev.Name = 'OO SO1_Rev'
        cons_S01_rev.Outside_Layer = 'Floor/Roof Screed_.O7'
        cons_S01_rev.Layer_2 = 'O SO1_.16'
        cons_S01_rev.Layer_3 = 'Cast Concrete_.1'
    else:
        idf1.newidfobject(
            'CONSTRUCTION',
            Name='OO SO1_Rev',
            Outside_Layer='Floor/Roof Screed_.O7',
            Layer_2='O SO1_.16',
            Layer_3='Cast Concrete_.1'
        )

    if 'OOO CTE Ladrillo perforado_.O5' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
        mat_perfbrick = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'OOO CTE Ladrillo perforado_.O5'][0]
        mat_perfbrick.Roughness = 'Rough'
        mat_perfbrick.Thickness = 0.05
        mat_perfbrick.Conductivity = 0.35
        mat_perfbrick.Density = 780
        mat_perfbrick.Specific_Heat = 1000
        mat_perfbrick.Thermal_Absorptance = 0.9
        mat_perfbrick.Solar_Absorptance = 0.7
        mat_perfbrick.Visible_Absorptance = 0.7
    else:
        idf1.newidfobject(
            'MATERIAL',
            Name='OOO CTE Ladrillo perforado_.O5',
            Roughness='Rough',
            Thickness=0.05,
            Conductivity=0.35,
            Density=780,
            Specific_Heat=1000,
            Thermal_Absorptance=0.9,
            Solar_Absorptance=0.7,
            Visible_Absorptance=0.7
        )

    if 'O FO1_.125' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
        mat_F01 = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'O FO1_.125'][0]
        mat_F01.Roughness = 'Rough'
        mat_F01.Thickness = 0.125
        mat_F01.Conductivity = 0.01292
        mat_F01.Density = 1700
        mat_F01.Specific_Heat = 1000
        mat_F01.Thermal_Absorptance = 0.9
        mat_F01.Solar_Absorptance = 0.7
        mat_F01.Visible_Absorptance = 0.7
    else:
        idf1.newidfobject(
            'MATERIAL',
            Name='O FO1_.125',
            Roughness='Rough',
            Thickness=0.125,
            Conductivity=0.01292,
            Density=1700,
            Specific_Heat=1000,
            Thermal_Absorptance=0.9,
            Solar_Absorptance=0.7,
            Visible_Absorptance=0.7
        )

    if 'OOO CTE Mortero_.O1' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
        mat_cementplaster = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'OOO CTE Mortero_.O1'][0]
        mat_cementplaster.Roughness = 'Rough'
        mat_cementplaster.Thickness = 0.01
        mat_cementplaster.Conductivity = 1
        mat_cementplaster.Density = 1700
        mat_cementplaster.Specific_Heat = 1000
        mat_cementplaster.Thermal_Absorptance = 0.9
        mat_cementplaster.Solar_Absorptance = 0.7
        mat_cementplaster.Visible_Absorptance = 0.7
    else:
        idf1.newidfobject(
            'MATERIAL',
            Name='OOO CTE Mortero_.O1',
            Roughness='Rough',
            Thickness=0.01,
            Conductivity=1,
            Density=1700,
            Specific_Heat=1000,
            Thermal_Absorptance=0.9,
            Solar_Absorptance=0.7,
            Visible_Absorptance=0.7
        )

    if 'OO FO1' in [i.Name for i in idf1.idfobjects['CONSTRUCTION']]:
        cons_F01 = [i for i in idf1.idfobjects['CONSTRUCTION'] if i.Name == 'OO FO1'][0]
        cons_F01.Name = 'OO FO1'
        cons_F01.Outside_Layer = 'OOO CTE Ladrillo perforado_.O5'
        cons_F01.Layer_2 = 'O FO1_.125'
        cons_F01.Layer_3 = 'OOO CTE Mortero_.O1'
    else:
        idf1.newidfobject(
            'CONSTRUCTION',
            Name='OO FO1',
            Outside_Layer='OOO CTE Ladrillo perforado_.O5',
            Layer_2='O FO1_.125',
            Layer_3='OOO CTE Mortero_.O1'
        )

    if 'OO FO1_Rev' in [i.Name for i in idf1.idfobjects['CONSTRUCTION']]:
        cons_F01_rev = [i for i in idf1.idfobjects['CONSTRUCTION'] if i.Name == 'OO FO1_Rev'][0]
        cons_F01_rev.Name = 'OO FO1_Rev'
        cons_F01_rev.Outside_Layer = 'OOO CTE Ladrillo perforado_.O5'
        cons_F01_rev.Layer_2 = 'O FO1_.125'
        cons_F01_rev.Layer_3 = 'OOO CTE Mortero_.O1'
    else:
        idf1.newidfobject(
            'CONSTRUCTION',
            Name='OO FO1_Rev',
            Outside_Layer='OOO CTE Mortero_.O1',
            Layer_2='O FO1_.125',
            Layer_3='OOO CTE Ladrillo perforado_.O5'
        )

    if 'OOO CTE Arena_.O25' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
        mat_sand = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'OOO CTE Arena_.O25'][0]
        mat_sand.Roughness = 'Rough'
        mat_sand.Thickness = 0.025
        mat_sand.Conductivity = 2
        mat_sand.Density = 1700
        mat_sand.Specific_Heat = 910
        mat_sand.Thermal_Absorptance = 0.9
        mat_sand.Solar_Absorptance = 0.7
        mat_sand.Visible_Absorptance = 0.7
    else:
        idf1.newidfobject(
            'MATERIAL',
            Name='OOO CTE Arena_.O25',
            Roughness='Rough',
            Thickness=0.025,
            Conductivity=2,
            Density=1700,
            Specific_Heat=910,
            Thermal_Absorptance=0.9,
            Solar_Absorptance=0.7,
            Visible_Absorptance=0.7
        )

    if 'O CO1_.2' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
        mat_C01 = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'O CO1_.2'][0]
        mat_C01.Roughness = 'Rough'
        mat_C01.Thickness = 0.2
        mat_C01.Conductivity = 0.03059
        mat_C01.Density = 1700
        mat_C01.Specific_Heat = 1000
        mat_C01.Thermal_Absorptance = 0.9
        mat_C01.Solar_Absorptance = 0.7
        mat_C01.Visible_Absorptance = 0.7
    else:
        idf1.newidfobject(
            'MATERIAL',
            Name='O CO1_.2',
            Roughness='Rough',
            Thickness=0.2,
            Conductivity=0.03059,
            Density=1700,
            Specific_Heat=1000,
            Thermal_Absorptance=0.9,
            Solar_Absorptance=0.7,
            Visible_Absorptance=0.7
        )

    if 'Cast Concrete_.3' in [mat.Name for mat in idf1.idfobjects['MATERIAL']]:
        mat_cast_concrete_thicker = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name == 'Cast Concrete_.3'][0]
        mat_cast_concrete_thicker.Roughness = 'Rough'
        mat_cast_concrete_thicker.Thickness = 0.3
        mat_cast_concrete_thicker.Conductivity = 1.13
        mat_cast_concrete_thicker.Density = 2000
        mat_cast_concrete_thicker.Specific_Heat = 1000
        mat_cast_concrete_thicker.Thermal_Absorptance = 0.9
        mat_cast_concrete_thicker.Solar_Absorptance = 0.6
        mat_cast_concrete_thicker.Visible_Absorptance = 0.6
    else:
        idf1.newidfobject(
            'MATERIAL',
            Name='Cast Concrete_.3',
            Roughness='Rough',
            Thickness=0.3,
            Conductivity=1.13,
            Density=2000,
            Specific_Heat=1000,
            Thermal_Absorptance=0.9,
            Solar_Absorptance=0.6,
            Visible_Absorptance=0.6
        )

    if 'OO CO1' in [i.Name for i in idf1.idfobjects['CONSTRUCTION']]:
        cons_C01 = [i for i in idf1.idfobjects['CONSTRUCTION'] if i.Name == 'OO CO1'][0]
        cons_C01.Name = 'OO CO1'
        cons_C01.Outside_Layer = 'OOO CTE Arena_.O25'
        cons_C01.Layer_2 = 'O CO1_.2'
        cons_C01.Layer_3 = 'Cast Concrete_.3'
    else:
        idf1.newidfobject(
            'CONSTRUCTION',
            Name='OO CO1',
            Outside_Layer='OOO CTE Arena_.O25',
            Layer_2='O CO1_.2',
            Layer_3='Cast Concrete_.3'
        )

    if 'OO CO1_Rev' in [i.Name for i in idf1.idfobjects['CONSTRUCTION']]:
        cons_C01_rev = [i for i in idf1.idfobjects['CONSTRUCTION'] if i.Name == 'OO CO1_Rev'][0]
        cons_C01_rev.Name = 'OO CO1_Rev'
        cons_C01_rev.Outside_Layer = 'Cast Concrete_.3'
        cons_C01_rev.Layer_2 = 'O CO1_.2'
        cons_C01_rev.Layer_3 = 'OOO CTE Arena_.O25'
    else:
        idf1.newidfobject(
            'CONSTRUCTION',
            Name='OO CO1_Rev',
            Outside_Layer='Cast Concrete_.3',
            Layer_2='O CO1_.2',
            Layer_3='OOO CTE Arena_.O25'
        )

    floors = [i for i in idf1.idfobjects['BuildingSurface:Detailed']
              if i.Surface_Type == 'Floor' and i.Outside_Boundary_Condition == 'Ground']
    # print(floors)
    for i in floors:
        i.Construction_Name = 'OO SO1'

    walls = [i for i in idf1.idfobjects['BuildingSurface:Detailed']
             if i.Surface_Type == 'Wall' and i.Outside_Boundary_Condition == 'Outdoors']
    print(walls)
    for i in walls:
        i.Construction_Name = 'OO FO1'

    roofs = [i for i in idf1.idfobjects['BuildingSurface:Detailed']
             if i.Surface_Type == 'Roof' and i.Outside_Boundary_Condition == 'Outdoors']
    # print(roofs)
    for i in roofs:
        i.Construction_Name = 'OO CO1'




    facade_mat = [mat
                  for mat
                  in idf1.idfobjects['MATERIAL']
                  if mat.Name == 'O FO1_.125'][0]
    # print(facade_mat)
    roof_mat = [mat
                for mat
                in idf1.idfobjects['MATERIAL']
                if mat.Name == 'O CO1_.2'][0]
    # print(roof_mat)
    try:
        floor_mat = [mat
                     for mat
                     in idf1.idfobjects['MATERIAL']
                     if mat.Name == 'O SO1_.16'][0]
        # print(floor_mat)
        therearefloors = True
    except IndexError:
        therearefloors = False
        # print(therearefloors)
    facade_cond = facade_mat.Conductivity
    # print(facade_cond)
    roof_cond = roof_mat.Conductivity
    # print(roof_cond)
    if therearefloors:
        floor_cond = floor_mat.Conductivity
        # print(floor_cond)
    # else:
    #     print('there are no floors')
    for i in range(len(facade_cond_list)):
        facade_mat.Conductivity = facade_cond_list[i]
        for j in range(len(roof_cond_list)):
            roof_mat.Conductivity = roof_cond_list[j]
            if therearefloors:
                floor_mat.Conductivity = floor_cond_list[j]
                if i+1 < 10:
                    if j+1 < 10:
                        outputname = (
                            filename
                            + '[F0'+repr(i+1)
                            + '[C0'+repr(j+1)
                            + '[S0'+repr(j+1)
                            + '.idf'
                        )
                    else:
                        outputname = (
                            filename
                            + '[F0'+repr(i+1)
                            + '[C'+repr(j+1)
                            + '[S'+repr(j+1)
                            + '.idf'
                        )
                else:
                    if j+1 < 10:
                        outputname = (
                            filename
                            + '[F'+repr(i+1)
                            + '[C0'+repr(j+1)
                            + '[S0'+repr(j+1)
                            + '.idf'
                        )
                    else:
                        outputname = (
                            filename
                            + '[F'+repr(i+1)
                            + '[C'+repr(j+1)
                            + '[S'+repr(j+1)
                            + '.idf'
                        )
            else:
                if i+1 < 10:
                    if j+1 < 10:
                        outputname = (
                            filename
                            + '[F0'+repr(i+1)
                            + '[C0'+repr(j+1)
                            + '[SXX'
                            + '.idf'
                        )
                    else:
                        outputname = (
                            filename
                            + '[F0'+repr(i+1)
                            + '[C'+repr(j+1)
                            + '[SXX'
                            + '.idf'
                        )
                else:
                    if j+1 < 10:
                        outputname = (
                            filename
                            + '[F'+repr(i+1)
                            + '[C0'+repr(j+1)
                            + '[SXX'
                            + '.idf'
                        )
                    else:
                        outputname = (
                            filename
                            + '[F'+repr(i+1)
                            + '[C'+repr(j+1)
                            + '[SXX'
                            + '.idf'
                        )
            idf1.savecopy(outputpath+'/'+outputname)



from eppy import modeleditor
from eppy.modeleditor import IDF
import os

iddfile = r"C:\EnergyPlusV9-4-0\Energy+.idd"
# fname1 = r"D:\OneDrive - UNIVERSIDAD DE SEVILLA\Papers OneDrive\VPO_cadiz_parametrico\BA_V01.idf"

path = r'C:\Users\daniel.sanchez\Documents\Personal\VPO_parametrico\input_IDFs'
outputpath = r'C:\Users\daniel.sanchez\Documents\Personal\VPO_parametrico\output_IDFs'

IDF.setiddname(iddfile)

file = path+'/BA_V01_corregido.idf'

idf1 = IDF(file)

# idf1.printidf()

C01 = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name== 'O CO1_.3'][0]
print(C01)

C01.Name = 'O CO1_.2'
C01.Thickness = 0.2

print(C01)

slab = [mat for mat in idf1.idfobjects['MATERIAL'] if mat.Name== 'OOO Hormigon celular_.1'][0]

print(slab)

slab.Name = 'Cast Concrete_.3'
slab.Thickness = 0.3
slab.Conductivity = 1.13
slab.Density = 2000
slab.Thermal_Absorptance = 0.9
slab.Solar_Absorptance = 0.6
slab.Visible_Absorptance = 0.6

print(slab)

buildup = [cons for cons in idf1.idfobjects['CONSTRUCTION'] if cons.Name == 'OO CO1'][0]

print(buildup)

buildup.Layer_2 = 'O CO1_.2'
buildup.Layer_3 = 'Cast Concrete_.3'

print(buildup)

buildup_rev = [cons for cons in idf1.idfobjects['CONSTRUCTION'] if cons.Name == 'OO CO1_Rev'][0]

print(buildup_rev)

buildup_rev.Outside_Layer = 'Cast Concrete_.3'
buildup_rev.Layer_2 = 'O CO1_.2'

print(buildup_rev)

idf1.savecopy(file.split('.idf')[0]+'_amended.idf')


"""Example of script usage with arguments."""

from accim.sim import accis

scriptTypeList = ['vrf_mm']
outputsFreqList = [['hourly'], ['daily'], ['monthly'], ['runperiod'], ['hourly', 'daily', 'monthly', 'runperiod']]
EPlist = ['22.2']

for i in scriptTypeList:
    for j in range(len(outputsFreqList)):
        for k in EPlist:
            accis.addAccis(
                ScriptType=i,
                Output_keep_existing=False,
                Output_type='standard',
                Output_freqs=outputsFreqList[j],
                EnergyPlus_version=k,
                TempCtrl='temp',
                ComfStand=[1],
                CAT=[1],
                ComfMod=[1],
                HVACmode=[2],
                VentCtrl=[0],
                VSToffset=[0],
                MinOToffset=[50],
                MaxWindSpeed=[50],
                ASTtol_start=0.1,
                ASTtol_end_input=0.1,
                ASTtol_steps=0.1,
                NameSuffix='_'.join(outputsFreqList[j])
            )



"""Add EnergyPlus objects in common to both ExistingHVAC and VRFsystem."""


def setComfFieldsPeople(self, EnergyPlus_version: str = None, TempCtrl: str = None, verboseMode: bool = True):
    """
    Amend PEOPLE objects so that accim can work.

    Copy existing PEOPLE objects and adds AdaptiveASH55 and AdaptiveCEN15251
    to Thermal Comfort Model types 1 and 2 fields of the existing People
    objects.
    """
    ppl = ([people for people in self.idf1.idfobjects['PEOPLE']])
    for i in range(len(ppl)):
        if TempCtrl == 'pmv':
            if EnergyPlus_version.lower() == '9.6' or '22' in EnergyPlus_version.lower():
                self.idf1.newidfobject(
                    'PEOPLE',
                    Name=ppl[i].Name,
                    Zone_or_ZoneList_or_Space_or_SpaceList_Name=ppl[i].Zone_or_ZoneList_or_Space_or_SpaceList_Name,
                    Number_of_People_Schedule_Name=ppl[i].Number_of_People_Schedule_Name,
                    Number_of_People_Calculation_Method=ppl[i].Number_of_People_Calculation_Method,
                    Number_of_People=ppl[i].Number_of_People,
                    People_per_Floor_Area=ppl[i].People_per_Floor_Area,
                    Floor_Area_per_Person=ppl[i].Floor_Area_per_Person,
                    Fraction_Radiant=ppl[i].Fraction_Radiant,
                    Sensible_Heat_Fraction=ppl[i].Sensible_Heat_Fraction,
                    Activity_Level_Schedule_Name=ppl[i].Activity_Level_Schedule_Name,
                    Carbon_Dioxide_Generation_Rate=ppl[i].Carbon_Dioxide_Generation_Rate,
                    Enable_ASHRAE_55_Comfort_Warnings=ppl[i].Enable_ASHRAE_55_Comfort_Warnings,
                    Mean_Radiant_Temperature_Calculation_Type=ppl\
                        [i].Mean_Radiant_Temperature_Calculation_Type,
                    Surface_NameAngle_Factor_List_Name=ppl[i].Surface_NameAngle_Factor_List_Name,
                    Work_Efficiency_Schedule_Name=ppl[i].Work_Efficiency_Schedule_Name,
                    Clothing_Insulation_Calculation_Method=ppl\
                        [i].Clothing_Insulation_Calculation_Method,
                    Clothing_Insulation_Calculation_Method_Schedule_Name=ppl\
                        [i].Clothing_Insulation_Calculation_Method_Schedule_Name,
                    Clothing_Insulation_Schedule_Name=ppl[i].Clothing_Insulation_Schedule_Name,
                    Air_Velocity_Schedule_Name=ppl[i].Air_Velocity_Schedule_Name,
                    Thermal_Comfort_Model_1_Type='AdaptiveASH55',
                    Thermal_Comfort_Model_2_Type='AdaptiveCEN15251',
                    Thermal_Comfort_Model_3_Type='Fanger',
                    Thermal_Comfort_Model_4_Type='',
                    Thermal_Comfort_Model_5_Type='',
                    )
                firstpeopleobject = self.idf1.idfobjects['PEOPLE'][0]
                self.idf1.removeidfobject(firstpeopleobject)
            else:
                self.idf1.newidfobject(
                    'PEOPLE',
                    Name=ppl[i].Name,
                    Zone_or_ZoneList_Name=ppl[i].Zone_or_ZoneList_Name,
                    Number_of_People_Schedule_Name=ppl[i].Number_of_People_Schedule_Name,
                    Number_of_People_Calculation_Method=ppl[i].Number_of_People_Calculation_Method,
                    Number_of_People=ppl[i].Number_of_People,
                    People_per_Zone_Floor_Area=ppl[i].People_per_Zone_Floor_Area,
                    Zone_Floor_Area_per_Person=ppl[i].Zone_Floor_Area_per_Person,
                    Fraction_Radiant=ppl[i].Fraction_Radiant,
                    Sensible_Heat_Fraction=ppl[i].Sensible_Heat_Fraction,
                    Activity_Level_Schedule_Name=ppl[i].Activity_Level_Schedule_Name,
                    Carbon_Dioxide_Generation_Rate=ppl[i].Carbon_Dioxide_Generation_Rate,
                    Enable_ASHRAE_55_Comfort_Warnings=ppl[i].Enable_ASHRAE_55_Comfort_Warnings,
                    Mean_Radiant_Temperature_Calculation_Type=ppl\
                        [i].Mean_Radiant_Temperature_Calculation_Type,
                    Surface_NameAngle_Factor_List_Name=ppl[i].Surface_NameAngle_Factor_List_Name,
                    Work_Efficiency_Schedule_Name=ppl[i].Work_Efficiency_Schedule_Name,
                    Clothing_Insulation_Calculation_Method=ppl\
                        [i].Clothing_Insulation_Calculation_Method,
                    Clothing_Insulation_Calculation_Method_Schedule_Name=ppl\
                        [i].Clothing_Insulation_Calculation_Method_Schedule_Name,
                    Clothing_Insulation_Schedule_Name=ppl[i].Clothing_Insulation_Schedule_Name,
                    Air_Velocity_Schedule_Name=ppl[i].Air_Velocity_Schedule_Name,
                    Thermal_Comfort_Model_1_Type='AdaptiveASH55',
                    Thermal_Comfort_Model_2_Type='AdaptiveCEN15251',
                    Thermal_Comfort_Model_3_Type='Fanger',
                    Thermal_Comfort_Model_4_Type='',
                    Thermal_Comfort_Model_5_Type='',
                    )
                firstpeopleobject = self.idf1.idfobjects['PEOPLE'][0]
                self.idf1.removeidfobject(firstpeopleobject)
        else:
            if EnergyPlus_version.lower() == '9.6' or '22' in EnergyPlus_version.lower():
                self.idf1.newidfobject(
                    'PEOPLE',
                    Name=ppl[i].Name,
                    Zone_or_ZoneList_or_Space_or_SpaceList_Name=ppl[i].Zone_or_ZoneList_or_Space_or_SpaceList_Name,
                    Number_of_People_Schedule_Name=ppl[i].Number_of_People_Schedule_Name,
                    Number_of_People_Calculation_Method=ppl[i].Number_of_People_Calculation_Method,
                    Number_of_People=ppl[i].Number_of_People,
                    People_per_Floor_Area=ppl[i].People_per_Floor_Area,
                    Floor_Area_per_Person=ppl[i].Floor_Area_per_Person,
                    Fraction_Radiant=ppl[i].Fraction_Radiant,
                    Sensible_Heat_Fraction=ppl[i].Sensible_Heat_Fraction,
                    Activity_Level_Schedule_Name=ppl[i].Activity_Level_Schedule_Name,
                    Carbon_Dioxide_Generation_Rate=ppl[i].Carbon_Dioxide_Generation_Rate,
                    Enable_ASHRAE_55_Comfort_Warnings=ppl[i].Enable_ASHRAE_55_Comfort_Warnings,
                    Mean_Radiant_Temperature_Calculation_Type=ppl\
                        [i].Mean_Radiant_Temperature_Calculation_Type,
                    Surface_NameAngle_Factor_List_Name=ppl[i].Surface_NameAngle_Factor_List_Name,
                    Work_Efficiency_Schedule_Name=ppl[i].Work_Efficiency_Schedule_Name,
                    Clothing_Insulation_Calculation_Method=ppl\
                        [i].Clothing_Insulation_Calculation_Method,
                    Clothing_Insulation_Calculation_Method_Schedule_Name=ppl\
                        [i].Clothing_Insulation_Calculation_Method_Schedule_Name,
                    Clothing_Insulation_Schedule_Name=ppl[i].Clothing_Insulation_Schedule_Name,
                    Air_Velocity_Schedule_Name=ppl[i].Air_Velocity_Schedule_Name,
                    Thermal_Comfort_Model_1_Type='AdaptiveASH55',
                    Thermal_Comfort_Model_2_Type='AdaptiveCEN15251',
                    Thermal_Comfort_Model_3_Type='',
                    Thermal_Comfort_Model_4_Type='',
                    Thermal_Comfort_Model_5_Type='',
                    )
                firstpeopleobject = self.idf1.idfobjects['PEOPLE'][0]
                self.idf1.removeidfobject(firstpeopleobject)
            else:
                self.idf1.newidfobject(
                    'PEOPLE',
                    Name=ppl[i].Name,
                    Zone_or_ZoneList_Name=ppl[i].Zone_or_ZoneList_Name,
                    Number_of_People_Schedule_Name=ppl[i].Number_of_People_Schedule_Name,
                    Number_of_People_Calculation_Method=ppl[i].Number_of_People_Calculation_Method,
                    Number_of_People=ppl[i].Number_of_People,
                    People_per_Zone_Floor_Area=ppl[i].People_per_Zone_Floor_Area,
                    Zone_Floor_Area_per_Person=ppl[i].Zone_Floor_Area_per_Person,
                    Fraction_Radiant=ppl[i].Fraction_Radiant,
                    Sensible_Heat_Fraction=ppl[i].Sensible_Heat_Fraction,
                    Activity_Level_Schedule_Name=ppl[i].Activity_Level_Schedule_Name,
                    Carbon_Dioxide_Generation_Rate=ppl[i].Carbon_Dioxide_Generation_Rate,
                    Enable_ASHRAE_55_Comfort_Warnings=ppl[i].Enable_ASHRAE_55_Comfort_Warnings,
                    Mean_Radiant_Temperature_Calculation_Type=ppl\
                        [i].Mean_Radiant_Temperature_Calculation_Type,
                    Surface_NameAngle_Factor_List_Name=ppl[i].Surface_NameAngle_Factor_List_Name,
                    Work_Efficiency_Schedule_Name=ppl[i].Work_Efficiency_Schedule_Name,
                    Clothing_Insulation_Calculation_Method=ppl\
                        [i].Clothing_Insulation_Calculation_Method,
                    Clothing_Insulation_Calculation_Method_Schedule_Name=ppl\
                        [i].Clothing_Insulation_Calculation_Method_Schedule_Name,
                    Clothing_Insulation_Schedule_Name=ppl[i].Clothing_Insulation_Schedule_Name,
                    Air_Velocity_Schedule_Name=ppl[i].Air_Velocity_Schedule_Name,
                    Thermal_Comfort_Model_1_Type='AdaptiveASH55',
                    Thermal_Comfort_Model_2_Type='AdaptiveCEN15251',
                    Thermal_Comfort_Model_3_Type='',
                    Thermal_Comfort_Model_4_Type='',
                    Thermal_Comfort_Model_5_Type='',
                    )
                firstpeopleobject = self.idf1.idfobjects['PEOPLE'][0]
                self.idf1.removeidfobject(firstpeopleobject)

    ppl = ([people for people in self.idf1.idfobjects['PEOPLE']])

    if len([i for i in self.idf1.idfobjects['zonelist']]) > 0:
        # zonelist = [i for i in self.idf1.idfobjects['zonelist']]
        # spacelist = [i for i in self.idf1.idfobjects['spacelist']]
        ppl = [i for i in self.idf1.idfobjects['people']]
        # todo if people zone or zonelist field is a zonelist, add a people object for a zone
        newppl = ppl[-1]
        newppl = self.idf1.copyidfobject(newppl)
        newppl.Name = self.occupiedZones_orig[0] + ' People'
        newppl.Zone_or_ZoneList_or_Space_or_SpaceList_Name = self.occupiedZones_orig[0]
        self.newppl = newppl

    if verboseMode:
        print('The people objects in the model have been amended.')
        # print(*peoplelist,sep="\n")
    del ppl, firstpeopleobject


def saveaccim(self, verboseMode: bool = True):
    """Save IDF."""
    self.idf1.save()
    if verboseMode:
        print('IDF has been saved')


def setPMVsetpoint(self, verboseMode: bool = True):
    """Sets PMV setpoints for temperature control."""
    # previoustodo check again the difference between operative temp and fanger; see ZoneControl:Thermostat and where is it assigned
    optempthermlist = ([program for program in self.idf1.idfobjects['ZoneControl:Thermostat:OperativeTemperature']])

    for i in range(len(optempthermlist)):
        firstoptempthermlist = self.idf1.idfobjects['ZoneControl:Thermostat:OperativeTemperature'][-1]
        self.idf1.removeidfobject(firstoptempthermlist)

    fangerdict = {
        'Cooling Fanger comfort setpoint: Always 0.5': '0.5',
        'Heating Fanger comfort setpoint: Always -0.5': '-0.5'
    }
    for i in fangerdict:
        if i in [schedule.Name for schedule in self.idf1.idfobjects['Schedule:Compact']]:
            if verboseMode:
                print(f"{i} Schedule already was in the model")
        else:
            self.idf1.newidfobject(
                'Schedule:Compact',
                Name=i,
                Schedule_Type_Limits_Name="Any Number",
                Field_1='Through: 12/31',
                Field_2='For: AllDays',
                Field_3='Until: 24:00,' + fangerdict[i]
                )
            if verboseMode:
                print(f"{i} Schedule has been added")

    for zone in self.zonenames_orig:
        if f'{zone} Comfort Control' in [i.Name for i in self.idf1.idfobjects['ZoneControl:Thermostat:ThermalComfort']]:
            if verboseMode:
                print(f'{zone} Comfort Control ZoneControl:Thermostat:ThermalComfort already was in the model')
        else:
            self.idf1.newidfobject(
                'ZoneControl:Thermostat:ThermalComfort',
                Name=f'{zone} Comfort Control',
                Zone_or_ZoneList_Name=zone,
                Averaging_Method='PeopleAverage',
                Minimum_DryBulb_Temperature_Setpoint=12.8,
                Maximum_DryBulb_Temperature_Setpoint=40.0,
                Thermal_Comfort_Control_Type_Schedule_Name='Zone Comfort Control Type Sched',
                Thermal_Comfort_Control_1_Object_Type='ThermostatSetpoint:ThermalComfort:Fanger:DualSetpoint',
                Thermal_Comfort_Control_1_Name=f'{zone} Dual Comfort Setpoint'
            )
            if verboseMode:
                print(f'{zone} Comfort Control ZoneControl:Thermostat:ThermalComfort has been added')

        if f'{zone} Dual Comfort Setpoint' in [i.Name for i in self.idf1.idfobjects['ThermostatSetpoint:ThermalComfort:Fanger:DualSetpoint']]:
            if verboseMode:
                print(f'{zone} Dual Comfort Setpoint ThermostatSetpoint:ThermalComfort:Fanger:DualSetpoint already was in the model')
        else:
            self.idf1.newidfobject(
                'ThermostatSetpoint:ThermalComfort:Fanger:DualSetpoint',
                Name=f'{zone} Dual Comfort Setpoint',
                Fanger_Thermal_Comfort_Heating_Schedule_Name='Heating Fanger comfort setpoint: Always -0.5',
                Fanger_Thermal_Comfort_Cooling_Schedule_Name='Cooling Fanger comfort setpoint: Always 0.5'
            )
            if verboseMode:
                print(f'{zone} Dual Comfort Setpoint ThermostatSetpoint:ThermalComfort:Fanger:DualSetpoint has been added')



from accim.sim import accis

def test_addAccis():
    from accim.sim import accis
    from os import listdir
    scriptTypeList = ['vrf']
    outputsList = ['simplified', 'standard', 'timestep']
    EPlist = ['ep95']

    for i in scriptTypeList:
        for j in outputsList:
            for k in EPlist:
                accis.addAccis(
                    ScriptType=i,
                    Output_type=j,
                    EnergyPlus_version=k,
                    ComfStand=[1],
                    CAT=[1],
                    ComfMod=[1],
                    HVACmode=[2],
                    VentCtrl=[0],
                    VSToffset=[0],
                    MinOToffset=[50],
                    MaxWindSpeed=[50],
                    ASTtol_start=0.1,
                    ASTtol_end_input=0.1,
                    ASTtol_steps=0.1,
                    NameSuffix=j,
                    verboseMode=False,
                    confirmGen=True
                )
    expectedNames = [
        'TestModel_VRFsystem_pymod[AS_EN16798[CA_1[CM_1[HM_2[VC_0[VO_0[MT_50[MW_50[AT_0.1[simplified.idf',
        'TestModel_VRFsystem_pymod[AS_EN16798[CA_1[CM_1[HM_2[VC_0[VO_0[MT_50[MW_50[AT_0.1[standard.idf',
        'TestModel_VRFsystem_pymod[AS_EN16798[CA_1[CM_1[HM_2[VC_0[VO_0[MT_50[MW_50[AT_0.1[timestep.idf'
        ]

    actualNames = [i for i in listdir() if i.endswith('.idf') and '_pymod' in i]

    for i in range(len(actualNames)):
        assert actualNames[i] == expectedNames[i]
